/**
 * graffiti.js - 
 * MIT License - (c) Roland Poulter 2011
 */
(function(){
var api = (function(){"use strict";
	var modules = {};
	return {
		require: function (path, from) {
			from = from || '/';
			var c = path.charAt(0);
			function rel () {
				if (from.charAt(from.length - 1) !== '/') {
					from += '/';
				}
				path = from + path;
			}
			if (c === '.') {
				c = path.charAt(1);
				if (c === '.' && path.charAt(2) === '/') {
					if (from.charAt(from.length - 1) === '/') {
						from = from.substring(0, from.length - 1);
					}
					from = from.substring(0, from.lastIndexOf('/'));
					path = from + path.substr(2);
				} else if (c === '/') {
					path = from + path.substr(2);
				} else {
					rel();
				}
			} else if (c !== '/') {
				rel();
			}
			if (path.charAt(0) !== '/') {
				path = '/' + path;
			}
			return modules[path].install();
		},
		provide: function (path, factory) {
			var req = this.require;
			modules[path] = {
				exports: {},
				factory: factory,
				require: function require (p) {
					var f = path.substring(0, path.lastIndexOf('/'));
					return req(p, f + '/');
				},
				install: function install () {
					var original = install;
					this.install = function (reset) {
						if (reset) return original.call(this);
						return this.exports;
					};
					this.factory(this.require, this, this.exports);
					return this.install();
				}
			};
			return this;
		},
		freeze: function (name, context) {
			delete this.provide;
			if (name && context) {
				this.conflict = context[name];
				this.noConflict = function () {
					return (context[name] = (this.conflict || this));
				};
				context[name] = this;
			}
			return this;
		}
	};
}).call(this);
api.provide("/glMath", function (require, module, exports) {
/**
 * glMath.js - WebGL math library with matrix and vector operations based on glMatrix.js
 * MIT License - (c) Roland Poulter 2011
 */
(function(){
var api = (function(){"use strict";
	var modules = {};
	return {
		require: function (path, from) {
			from = from || '/';
			var c = path.charAt(0);
			function rel () {
				if (from.charAt(from.length - 1) !== '/') {
					from += '/';
				}
				path = from + path;
			}
			if (c === '.') {
				c = path.charAt(1);
				if (c === '.' && path.charAt(2) === '/') {
					if (from.charAt(from.length - 1) === '/') {
						from = from.substring(0, from.length - 1);
					}
					from = from.substring(0, from.lastIndexOf('/'));
					path = from + path.substr(2);
				} else if (c === '/') {
					path = from + path.substr(2);
				} else {
					rel();
				}
			} else if (c !== '/') {
				rel();
			}
			if (path.charAt(0) !== '/') {
				path = '/' + path;
			}
			return modules[path].install();
		},
		provide: function (path, factory) {
			var req = this.require;
			modules[path] = {
				exports: {},
				factory: factory,
				require: function require (p) {
					var f = path.substring(0, path.lastIndexOf('/'));
					return req(p, f + '/');
				},
				install: function install () {
					var original = install;
					this.install = function (reset) {
						if (reset) return original.call(this);
						return this.exports;
					};
					this.factory(this.require, this, this.exports);
					return this.install();
				}
			};
			return this;
		},
		freeze: function (name, context) {
			delete this.provide;
			if (name && context) {
				this.conflict = context[name];
				this.noConflict = function () {
					return (context[name] = (this.conflict || this));
				};
				context[name] = this;
			}
			return this;
		}
	};
}).call(this);
api.provide("/utils", function (require, module, exports) {
/*global exports Float64Array*/
(function(){"use strict";
	var utils = exports;
	utils.degToRad = function (deg) {
		return deg * Math.PI / 180;
	};
	utils.hexToRGB = function (hex) {
		hex = hex.toString(16);
		var r = parseInt(hex.substring(0, 2), 16),
		    g = parseInt(hex.substring(2, 4), 16),
		    b = parseInt(hex.substring(4, 6), 16);
		return [r, g, b];
	};
	utils.rgbToHex = function (r, g, b) {
		return utils.toHex(r) + utils.toHex(g) + utils.toHex(b);
	};
	utils.toHex = function (n) {
		if (n === undefined) return '00';
		n = parseInt(n, 10);
		if (n === 0 || isNaN(n)) return '00';
		n = Math.max(0, n);
		n = Math.min(n, 255);
		n = Math.round(n);
		var chars = '0123456789ABCDEF';
		return chars.charAt((n - n % 16) / 16) +
			chars.charAt(n % 16);
	};
	utils.square = function (n) {
		return n * n;
	};
	utils.hypot = function (a, b) {
		return Math.sqrt(a*a + b*b);
	};
	utils.polar = function (x, y) {
		var theta = Math.atan2(y, x),
		    r = utils.hypot(x, y);

		return new Float64Array([r, theta]);
	};
	utils.cartesian = function (r, theta) {
		var x = r * Math.cos(theta),
		    y = r * Math.sin(theta);

		return new Float64Array([x, y]);
	};
}).call(this);
});
api.provide("/Vector3", function (require, module, exports) {
/*global module Float32Array Float64Array*/
(function(){"use strict";
	function Vector3 (vec) {return this.initVector3(vec);}
	var vector3 = Vector3.prototype;
	Vector3.check = function (obj) {
		return obj && obj.initVector3 === vector3.initVector3;
	};
	Vector3.create = function (obj, args) {
		obj = obj || [];
		obj.length = 3;
		obj.initVector3 = vector3.initVector3 || function (vec) {
			vec = vec || [];
			this[0] = vec[0] || 0;
			this[1] = vec[1] || 0;
			this[2] = vec[2] || 0;
			return this;
		};
		obj.set = function (vec) {
			return this.initVector3(vec);
		};
		obj.copy = function (vec) {
			return new Vector3(this);
		};
		obj.add = function (vec) {
			this[0] += vec[0];
			this[1] += vec[1];
			this[2] += vec[2];
			return this;
		};
		obj.subtract = function (vec) {
			this[0] -= vec[0];
			this[1] -= vec[1];
			this[2] -= vec[2];
			return this;
		};
		obj.negate = function (vec) {
			this[0] = -this[0];
			this[1] = -this[1];
			this[2] = -this[2];
			return this;
		};
		obj.multiply = function (vec) {
			this[0] *= vec[0];
			this[1] *= vec[1];
			this[2] *= vec[2];
			return this;
		};
		obj.divide = function (vec) {
			this[0] /= vec[0];
			this[1] /= vec[1];
			this[2] /= vec[2];
			return this;
		};
		obj.scale = function (val) {
			this[0] *= val;
			this[1] *= val;
			this[2] *= val;
			return this;
		};
		obj.zero = function () {
			this[0] = this[1] = this[2] = 0;
			return this;
		};
		obj.normalize = function () {
			var len = this.len();
			if (!len) this.zero();
			else if (len !== 1) {
				len = 1 / len;
				this[0] *= len;
				this[1] *= len;
				this[2] *= len;
			}
			return this;
		};
		obj.direction = function (vec) {
			var sub = this.copy().subtract(vec),
			    len = sub.len();
			if (!len) this.zero();
			else {
				len = 1 / len;
				this[0] = sub[0] * len;
				this[1] = sub[1] * len;
				this[2] = sub[2] * len;
			}
			return this;
		};
		obj.len = function () {
			return Math.sqrt(this.dot(this));
		};
		obj.dot = function (vec) {
			return this[0]*vec[0] + this[1]*vec[1] + this[2]*vec[2];
		};
		obj.cross = function (vec) {
			var  x = this[0], y = this[1], z = this[2],
			    x2 = vec[0], y2 = vec[1], z2 = vec[2];
			this[0] = y*z2 - z*y2;
			this[1] = z*x2 - x*z2;
			this[2] = x*y2 - y*x2;
			return this;
		};
		obj.lerp = function (vec, lerp) {
			var x = this[0], y = this[1], z = this[2];
			this[0] = x + lerp * (vec[0] - x);
			this[1] = y + lerp * (vec[1] - y);
			this[2] = z + lerp * (vec[2] - z);
			return this;
		};
		obj.toString = obj.str = function (vec) {
			return this.toArray().join(',');
		};
		obj.toArray = function () {
			return [this[0], this[1], this[2]];
		};
		obj.toFloat32Array = function () {
			return new Float32Array(this.toArray());
		};
		obj.toFloat64Array = function () {
			return new Float64Array(this.toArray());
		};
		return obj;
	};
	(module.exports = Vector3).create(vector3);
}).call(this);
});
api.provide("/Vector4", function (require, module, exports) {
/*global module require*/
(function(){"use strict";
	function Vector4 (vec) {return this.initVector4(vec);}
	var Vector3 = require('./Vector3'),
	    vector4 = Vector4.prototype;
	Vector4.check = function (obj) {
		return Vector3.check(obj) && obj.initVector4 === vector4.initVector4;
	};
	Vector4.create = function (obj, args) {
		obj = obj || [];
		if (!Vector3.check(obj)) Vector3.create(obj);
		obj.length = 4;
		obj.initVector4 = vector4.initVector4 || function (vec) {
			vec = vec || [];
			this[0] = vec[0] || 0;
			this[1] = vec[1] || 0;
			this[2] = vec[2] || 0;
			this[3] = vec[3] || 0;
			return this;
		};
		obj.set = function (vec) {
			return this.initVector4(vec);
		};
		obj.copy = function (vec) {
			return new Vector4(this);
		};
		obj.toArray = function () {
			return [this[0], this[1], this[2], this[4]];
		};
		if (args) obj.initVector4.apply(obj, args);
		return obj;
	};
	(module.exports = Vector4).create(vector4);
}).call(this);
});
api.provide("/Matrix3", function (require, module, exports) {
/*global module Float32Array Float64Array*/
(function(){"use strict";
	function Matrix3 (mat) {return this.initMatrix3(mat);}
	var matrix3 = Matrix3.prototype;
	Matrix3.check = function (obj) {
		return obj && obj.initMatrix3 === matrix3.initMatrix3;
	};
	Matrix3.create = function (obj, args) {
		obj = obj || [];
		obj.length = 9;
		obj.initMatrix3 = matrix3.initMatrix3 || function (mat) {
			mat = mat || [];
			this[0] = mat[0] || 0;
			this[1] = mat[1] || 0;
			this[2] = mat[2] || 0;
			this[3] = mat[3] || 0;
			this[4] = mat[4] || 0;
			this[5] = mat[5] || 0;
			this[6] = mat[6] || 0;
			this[7] = mat[7] || 0;
			this[8] = mat[8] || 0;
			return this;
		};
		obj.set = function (mat) {
			return this.initMatrix3(mat);
		};
		obj.copy = function () {
			return new Matrix3(this);
		};
		obj.identity = function () {
			return this.set([
				1, 0, 0,
				0, 1, 0,
				0, 0, 1
			]);
		};
		obj.transpose = function (mat) {
			var a01, a02, a12;
			if (!mat) {
				a01 = this[1];
				a02 = this[2];
				a12 = this[5];
				this[1] = this[3];
				this[2] = this[6];
				this[3] = a01;
				this[5] = this[7];
				this[6] = a02;
				this[7] = a12;
			} else {
				this[0] = mat[0];
				this[1] = mat[3];
				this[2] = mat[6];
				this[3] = mat[1];
				this[4] = mat[4];
				this[5] = mat[7];
				this[6] = mat[2];
				this[7] = mat[5];
				this[8] = mat[8];
			}
			return this;
		};
		obj.forEach = function (fn, bind) {
			if (typeof fn !== 'function') return this;
			bind = bind || this;
			for (var i = 0, l = this.length; i < l; i += 1) {
				fn.call(bind, this[i], i, this);
			}
			return this;
		};
		obj.toMat4 = function () {
			return [
				this[0], this[1], this[2], 0,
				this[3], this[4], this[5], 0,
				this[6], this[7], this[8], 0,
				      0,       0,       0, 1
			];
		};
		obj.toString = obj.str = function () {
			return this.toArray().join(',');
		};
		obj.toArray = function () {
			return [
				this[0], this[1], this[2],
				this[3], this[4], this[5],
				this[6], this[7], this[8]
			];
		};
		obj.toFloat32Array = function () {
			return new Float32Array(this.toArray());
		};
		obj.toFloat64Array = function () {
			return new Float64Array(this.toArray());
		};
		if (args) obj.initMatrix3.apply(obj, args);
		return obj;
	};
	(module.exports = Matrix3).create(matrix3);
}).call(this);
});
api.provide("/Matrix4", function (require, module, exports) {
/*global module require*/
(function(){"use strict";
	function Matrix4 (mat) {return this.initMatrix4(mat);}
	var Matrix3 = require('./Matrix3'),
	    matrix4 = Matrix4.prototype;
	Matrix4.check = function (obj) {
		return Matrix3.check(obj) && obj.initMatrix4 === matrix4.initMatrix4;
	};
	Matrix4.create = function (obj, args) {
		obj = obj || [];
		if (!Matrix3.check(obj)) Matrix3.create(obj);
		obj.length = 16;
		obj.initMatrix4 = matrix4.initMatrix4 || function (mat) {
			mat = mat || [];
			this[ 0] = mat[ 0] || 0;
			this[ 1] = mat[ 1] || 0;
			this[ 2] = mat[ 2] || 0;
			this[ 3] = mat[ 3] || 0;
			this[ 4] = mat[ 4] || 0;
			this[ 5] = mat[ 5] || 0;
			this[ 6] = mat[ 6] || 0;
			this[ 7] = mat[ 7] || 0;
			this[ 8] = mat[ 8] || 0;
			this[ 9] = mat[ 9] || 0;
			this[10] = mat[10] || 0;
			this[11] = mat[11] || 0;
			this[12] = mat[12] || 0;
			this[13] = mat[13] || 0;
			this[14] = mat[14] || 0;
			this[15] = mat[15] || 0;
			return this;
		};
		obj.set = function (mat) {
			return this.initMatrix4(mat);
		};
		obj.copy = function () {
			return new Matrix4(this);
		};
		obj.identity = function () {
			return this.set([
				1, 0, 0, 0,
				0, 1, 0, 0,
				0, 0, 1, 0,
				0, 0, 0, 1
			]);
		};
		obj.transpose = function (mat) {
			var a01, a02, a03, a12, a13, a23;
			if (!mat) {
				a01 = this[ 1];
				a02 = this[ 2];
				a03 = this[ 3];
				a12 = this[ 6];
				a13 = this[ 7];
				a23 = this[11];
				this[1] = this[4];
				this[2] = this[8];
				this[3] = this[12];
				this[4] = a01;
				this[6] = this[9];
				this[7] = this[13];
				this[8] = a02;
				this[9] = a12;
				this[11] = this[14];
				this[12] = a03;
				this[13] = a13;
				this[14] = a23;
			} else {
				this[0] = mat[0];
				this[1] = mat[4];
				this[2] = mat[8];
				this[3] = mat[12];
				this[4] = mat[1];
				this[5] = mat[5];
				this[6] = mat[9];
				this[7] = mat[13];
				this[8] = mat[2];
				this[9] = mat[6];
				this[10] = mat[10];
				this[11] = mat[14];
				this[12] = mat[3];
				this[13] = mat[7];
				this[14] = mat[11];
				this[15] = mat[15];
			}
			return this;
		};
		obj.determinant = function () {
			var a00 = this[ 0], a01 = this[ 1], a02 = this[ 2], a03 = this[ 3],
			    a10 = this[ 4], a11 = this[ 5], a12 = this[ 6], a13 = this[ 7],
			    a20 = this[ 8], a21 = this[ 9], a22 = this[10], a23 = this[11],
			    a30 = this[12], a31 = this[13], a32 = this[14], a33 = this[15];
			return a30*a21*a12*a03 - a20*a31*a12*a03 - a30*a11*a22*a03 + a10*a31*a22*a03 +
			       a20*a11*a32*a03 - a10*a21*a32*a03 - a30*a21*a02*a13 + a20*a31*a02*a13 +
			       a30*a01*a22*a13 - a00*a31*a22*a13 - a20*a01*a32*a13 + a00*a21*a32*a13 +
			       a30*a11*a02*a23 - a10*a31*a02*a23 - a30*a01*a12*a23 + a00*a31*a12*a23 +
			       a10*a01*a32*a23 - a00*a11*a32*a23 - a20*a11*a02*a33 + a10*a21*a02*a33 +
			       a20*a01*a12*a33 - a00*a21*a12*a33 - a10*a01*a22*a33 + a00*a11*a22*a33;
		};
		obj.inverse = function () {
			var a00 = this[ 0], a01 = this[ 1], a02 = this[ 2], a03 = this[ 3],
			    a10 = this[ 4], a11 = this[ 5], a12 = this[ 6], a13 = this[ 7],
			    a20 = this[ 8], a21 = this[ 9], a22 = this[10], a23 = this[11],
			    a30 = this[12], a31 = this[13], a32 = this[14], a33 = this[15],
			    b00 = a00*a11 - a01*a10,
			    b01 = a00*a12 - a02*a10,
			    b02 = a00*a13 - a03*a10,
			    b03 = a01*a12 - a02*a11,
			    b04 = a01*a13 - a03*a11,
			    b05 = a02*a13 - a03*a12,
			    b06 = a20*a31 - a21*a30,
			    b07 = a20*a32 - a22*a30,
			    b08 = a20*a33 - a23*a30,
			    b09 = a21*a32 - a22*a31,
			    b10 = a21*a33 - a23*a31,
			    b11 = a22*a33 - a23*a32,
			    invDet = 1/(b00*b11 - b01*b10 + b02*b09 + b03*b08 - b04*b07 + b05*b06);
			this[ 0] = ( a11*b11 - a12*b10 + a13*b09)*invDet;
			this[ 1] = (-a01*b11 + a02*b10 - a03*b09)*invDet;
			this[ 2] = ( a31*b05 - a32*b04 + a33*b03)*invDet;
			this[ 3] = (-a21*b05 + a22*b04 - a23*b03)*invDet;
			this[ 4] = (-a10*b11 + a12*b08 - a13*b07)*invDet;
			this[ 5] = ( a00*b11 - a02*b08 + a03*b07)*invDet;
			this[ 6] = (-a30*b05 + a32*b02 - a33*b01)*invDet;
			this[ 7] = ( a20*b05 - a22*b02 + a23*b01)*invDet;
			this[ 8] = ( a10*b10 - a11*b08 + a13*b06)*invDet;
			this[ 9] = (-a00*b10 + a01*b08 - a03*b06)*invDet;
			this[10] = ( a30*b04 - a31*b02 + a33*b00)*invDet;
			this[11] = (-a20*b04 + a21*b02 - a23*b00)*invDet;
			this[12] = (-a10*b09 + a11*b07 - a12*b06)*invDet;
			this[13] = ( a00*b09 - a01*b07 + a02*b06)*invDet;
			this[14] = (-a30*b03 + a31*b01 - a32*b00)*invDet;
			this[15] = ( a20*b03 - a21*b01 + a22*b00)*invDet;
			return this;
		};
		obj.toRotationMat = function () {
			return [
				this[0], this[1], this[ 2], this[ 3],
				this[4], this[5], this[ 6], this[ 7],
				this[8], this[9], this[10], this[11],
				      0,       0,        0,        1
			];
		};
		obj.toMat3 = function () {
			return [
				this[0], this[1], this[ 2],
				this[4], this[5], this[ 6],
				this[8], this[9], this[10]
			];
		};
		obj.toInverseMat3 = function () {
			var a00 = this[0], a01 = this[1], a02 = this[ 2],
			    a10 = this[4], a11 = this[5], a12 = this[ 6],
			    a20 = this[8], a21 = this[9], a22 = this[10],
			    b01 =  a22*a11 - a12*a21,
			    b11 = -a22*a10 + a12*a20,
			    b21 =  a21*a10 - a11*a20,
			    d = a00*b01 + a01*b11 + a02*b21,
			    id;
			if (!d) return null;
			id = 1/d;
			return [
				b01*id,
				(-a22*a01 + a02*a21)*id,
				(a12*a01 - a02*a11)*id,
				b11*id,
				(a22*a00 - a02*a20)*id,
				(-a12*a00 + a02*a10)*id,
				b21*id,
				(-a21*a00 + a01*a20)*id,
				(a11*a00 - a01*a10)*id
			];
		};
		obj.multiply = function (mat) {
			var a00 = this[ 0], a01 = this[ 1], a02 = this[ 2], a03 = this[ 3],
			    a10 = this[ 4], a11 = this[ 5], a12 = this[ 6], a13 = this[ 7],
			    a20 = this[ 8], a21 = this[ 9], a22 = this[10], a23 = this[11],
			    a30 = this[12], a31 = this[13], a32 = this[14], a33 = this[15],
			    b00 =  mat[ 0], b01 =  mat[ 1], b02 =  mat[ 2], b03 =  mat[ 3],
			    b10 =  mat[ 4], b11 =  mat[ 5], b12 =  mat[ 6], b13 =  mat[ 7],
			    b20 =  mat[ 8], b21 =  mat[ 9], b22 =  mat[10], b23 =  mat[11],
			    b30 =  mat[12], b31 =  mat[13], b32 =  mat[14], b33 =  mat[15];
			this[0] = b00*a00 + b01*a10 + b02*a20 + b03*a30;
			this[1] = b00*a01 + b01*a11 + b02*a21 + b03*a31;
			this[2] = b00*a02 + b01*a12 + b02*a22 + b03*a32;
			this[3] = b00*a03 + b01*a13 + b02*a23 + b03*a33;
			this[4] = b10*a00 + b11*a10 + b12*a20 + b13*a30;
			this[5] = b10*a01 + b11*a11 + b12*a21 + b13*a31;
			this[6] = b10*a02 + b11*a12 + b12*a22 + b13*a32;
			this[7] = b10*a03 + b11*a13 + b12*a23 + b13*a33;
			this[8] = b20*a00 + b21*a10 + b22*a20 + b23*a30;
			this[9] = b20*a01 + b21*a11 + b22*a21 + b23*a31;
			this[10] = b20*a02 + b21*a12 + b22*a22 + b23*a32;
			this[11] = b20*a03 + b21*a13 + b22*a23 + b23*a33;
			this[12] = b30*a00 + b31*a10 + b32*a20 + b33*a30;
			this[13] = b30*a01 + b31*a11 + b32*a21 + b33*a31;
			this[14] = b30*a02 + b31*a12 + b32*a22 + b33*a32;
			this[15] = b30*a03 + b31*a13 + b32*a23 + b33*a33;
			return this;
		};
		obj.multiplyVec3 = function (vec) {
			var x = vec[0], y = vec[1], z = vec[2];
			vec[0] = this[0]*x + this[4]*y + this[ 8]*z + this[12];
			vec[1] = this[1]*x + this[5]*y + this[ 9]*z + this[13];
			vec[2] = this[2]*x + this[6]*y + this[10]*z + this[14];
			return this;
		};
		obj.multiplyVec4 = function (vec) {
			var x = vec[0], y = vec[1], z = vec[2], w = vec[3];
			vec[0] = this[0]*x + this[4]*y + this[ 8]*z + this[12]*w;
			vec[1] = this[1]*x + this[5]*y + this[ 9]*z + this[13]*w;
			vec[2] = this[2]*x + this[6]*y + this[10]*z + this[14]*w;
			vec[3] = this[3]*x + this[7]*y + this[11]*z + this[15]*w;
			return this;
		};
		obj.translate = function (vec) {
			var x = vec[0], y = vec[1], z = vec[2];
			this[12] = this[0]*x + this[4]*y + this[ 8]*z + this[12];
			this[13] = this[1]*x + this[5]*y + this[ 9]*z + this[13];
			this[14] = this[2]*x + this[6]*y + this[10]*z + this[14];
			this[15] = this[3]*x + this[7]*y + this[11]*z + this[15];
			return this;
		};
		obj.scale = function (vec) {
			var x = vec[0], y = vec[1], z = vec[2];
			this[ 0] *= x;
			this[ 1] *= x;
			this[ 2] *= x;
			this[ 3] *= x;
			this[ 4] *= y;
			this[ 5] *= y;
			this[ 6] *= y;
			this[ 7] *= y;
			this[ 8] *= z;
			this[ 9] *= z;
			this[10] *= z;
			this[11] *= z;
			return this;
		};
		obj.rotate = function (angle, axis) {
			var x = axis[0],
			    y = axis[1],
			    z = axis[2],
			    len = Math.sqrt(x*x + y*y + z*z);
			if (!len) return null;
			if (len !== 1) {
				len = 1 / len;
				x *= len; 
				y *= len; 
				z *= len;
			}
			var s = Math.sin(angle), c = Math.cos(angle), t = 1 - c,
			    a00 = this[0], a01 = this[1], a02 = this[ 2], a03 = this[ 3],
			    a10 = this[4], a11 = this[5], a12 = this[ 6], a13 = this[ 7],
			    a20 = this[8], a21 = this[9], a22 = this[10], a23 = this[11],
			    b00 = x*x*t + c,   b01 = y*x*t + z*s, b02 = z*x*t - y*s,
			    b10 = x*y*t - z*s, b11 = y*y*t + c,   b12 = z*y*t + x*s,
			    b20 = x*z*t + y*s, b21 = y*z*t - x*s, b22 = z*z*t + c;
			this[ 0] = a00*b00 + a10*b01 + a20*b02;
			this[ 1] = a01*b00 + a11*b01 + a21*b02;
			this[ 2] = a02*b00 + a12*b01 + a22*b02;
			this[ 3] = a03*b00 + a13*b01 + a23*b02;
			this[ 4] = a00*b10 + a10*b11 + a20*b12;
			this[ 5] = a01*b10 + a11*b11 + a21*b12;
			this[ 6] = a02*b10 + a12*b11 + a22*b12;
			this[ 7] = a03*b10 + a13*b11 + a23*b12;
			this[ 8] = a00*b20 + a10*b21 + a20*b22;
			this[ 9] = a01*b20 + a11*b21 + a21*b22;
			this[10] = a02*b20 + a12*b21 + a22*b22;
			this[11] = a03*b20 + a13*b21 + a23*b22;
			return this;
		};
		obj.rotateX = function (angle) {
			var s = Math.sin(angle), c = Math.cos(angle),
			    a10 = this[4], a11 = this[5], a12 = this[ 6], a13 = this[ 7],
			    a20 = this[8], a21 = this[9], a22 = this[10], a23 = this[11];
			this[ 4] = a10*c + a20*s;
			this[ 5] = a11*c + a21*s;
			this[ 6] = a12*c + a22*s;
			this[ 7] = a13*c + a23*s;
			this[ 8] = a10*-s + a20*c;
			this[ 9] = a11*-s + a21*c;
			this[10] = a12*-s + a22*c;
			this[11] = a13*-s + a23*c;
			return this;
		};
		obj.rotateY = function (angle) {
			var s = Math.sin(angle), c = Math.cos(angle),
			    a00 = this[0], a01 = this[1], a02 = this[ 2], a03 = this[ 3],
			    a20 = this[8], a21 = this[9], a22 = this[10], a23 = this[11];
			this[ 0] = a00*c + a20*-s;
			this[ 1] = a01*c + a21*-s;
			this[ 2] = a02*c + a22*-s;
			this[ 3] = a03*c + a23*-s;
			this[ 8] = a00*s + a20*c;
			this[ 9] = a01*s + a21*c;
			this[10] = a02*s + a22*c;
			this[11] = a03*s + a23*c;
			return this;
		};
		obj.rotateZ = function (angle) {
			var s = Math.sin(angle), c = Math.cos(angle),
			    a00 = this[0], a01 = this[1], a02 = this[2], a03 = this[3],
			    a10 = this[4], a11 = this[5], a12 = this[6], a13 = this[7];
			this[0] = a00*c + a10*s;
			this[1] = a01*c + a11*s;
			this[2] = a02*c + a12*s;
			this[3] = a03*c + a13*s;
			this[4] = a00*-s + a10*c;
			this[5] = a01*-s + a11*c;
			this[6] = a02*-s + a12*c;
			this[7] = a03*-s + a13*c;
			return this;
		};
		obj.frustum = function (left, right, bottom, top, near, far) {
			var rl = (right - left),
			    tb = (top - bottom),
			    fn = (far - near);
			this[ 0] = (near*2) / rl;
			this[ 1] = 0;
			this[ 2] = 0;
			this[ 3] = 0;
			this[ 4] = 0;
			this[ 5] = (near*2) / tb;
			this[ 6] = 0;
			this[ 7] = 0;
			this[ 8] = (right + left) / rl;
			this[ 9] = (top + bottom) / tb;
			this[10] = -(far + near) / fn;
			this[11] = -1;
			this[12] = 0;
			this[13] = 0;
			this[14] = -(far*near*2) / fn;
			this[15] = 0;
			return this;
		};
		obj.perspective = function (fovy, aspect, near, far) {
			var top = near*Math.tan(fovy*Math.PI / 360.0),
			    right = top*aspect;
			return this.frustum(-right, right, -top, top, near, far);
		};
		obj.ortho = function (left, right, bottom, top, near, far) {
			var rl = (right - left),
			    tb = (top - bottom),
			    fn = (far - near);
			this[ 0] = 2 / rl;
			this[ 1] = 0;
			this[ 2] = 0;
			this[ 3] = 0;
			this[ 4] = 0;
			this[ 5] = 2 / tb;
			this[ 6] = 0;
			this[ 7] = 0;
			this[ 8] = 0;
			this[ 9] = 0;
			this[10] = -2 / fn;
			this[11] = 0;
			this[12] = -(left + right) / rl;
			this[13] = -(top + bottom) / tb;
			this[14] = -(far + near) / fn;
			this[15] = 1;
			return this;
		};
		obj.lookAt = function (eye, target, up) {
			var eyex = eye[0],
			    eyey = eye[1],
			    eyez = eye[2],
			    upx = up[0],
			    upy = up[1],
			    upz = up[2],
			    targetx = target[0],
			    targety = target[1],
			    targetz = target[2],
			    z0, z1, z2,
			    x0, x1, x2,
			    y0, y1, y2,
			    len;
			if (eyex === targetx && eyey === targety && eyez === targetz) {
				return this.identity();
			}
			z0 = eyex - target[0];
			z1 = eyey - target[1];
			z2 = eyez - target[2];
			len = 1/Math.sqrt(z0*z0 + z1*z1 + z2*z2);
			z0 *= len;
			z1 *= len;
			z2 *= len;
			x0 = upy*z2 - upz*z1;
			x1 = upz*z0 - upx*z2;
			x2 = upx*z1 - upy*z0;
			len = Math.sqrt(x0*x0 + x1*x1 + x2*x2);
			if (!len) {
				x0 = 0;
				x1 = 0;
				x2 = 0;
			} else {
				len = 1/len;
				x0 *= len;
				x1 *= len;
				x2 *= len;
			}
			y0 = z1*x2 - z2*x1;
			y1 = z2*x0 - z0*x2;
			y2 = z0*x1 - z1*x0;
			len = Math.sqrt(y0*y0 + y1*y1 + y2*y2);
			if (!len) {
				y0 = 0;
				y1 = 0;
				y2 = 0;
			} else {
				len = 1/len;
				y0 *= len;
				y1 *= len;
				y2 *= len;
			}
			this[0] = x0;
			this[1] = y0;
			this[2] = z0;
			this[3] = 0;
			this[4] = x1;
			this[5] = y1;
			this[6] = z1;
			this[7] = 0;
			this[8] = x2;
			this[9] = y2;
			this[10] = z2;
			this[11] = 0;
			this[12] = -(x0*eyex + x1*eyey + x2*eyez);
			this[13] = -(y0*eyex + y1*eyey + y2*eyez);
			this[14] = -(z0*eyex + z1*eyey + z2*eyez);
			this[15] = 1;
			return this;
		};
		obj.toArray = function () {
			return [
				this[ 0], this[ 1], this[ 2], this[ 3],
				this[ 4], this[ 5], this[ 6], this[ 7],
				this[ 8], this[ 9], this[10], this[11],
				this[12], this[13], this[14], this[15]
			];
		};
		if (args) obj.initMatrix4.apply(obj, args);
		return obj;
	};
	(module.exports = Matrix4).create(matrix4);
}).call(this);
});
api.provide("/Quaternion", function (require, module, exports) {
/*global module Float32Array Float64Array*/
(function (){"use strict";
	function Quaternion (quat) {return this.initQuaternion(quat);}
	var quaternion = Quaternion.prototype;
	Quaternion.check = function (obj) {
		return obj && obj.initQuaternion === quaternion.initQuaternion;
	};
	Quaternion.create = function (obj) {
		obj = obj || [];
		obj.length = 4;
		obj.initQuaternion = quaternion.initQuaternion || function (quat) {
			quat = quat || [];
			this[0] = quat[0] || 0;
			this[1] = quat[1] || 0;
			this[2] = quat[2] || 0;
			this[3] = quat[3] || 0;
			return this;
		};
		obj.set = function (quat) {
			return this.initQuaternion(quat);
		};
		obj.copy = function () {
			return new Quaternion(this);
		};
		obj.calculateW = function () {
			var x = this[0], y = this[1], z = this[2];
			this[3] = -Math.sqrt(Math.abs(1.0 - x*x - y*y - z*z));
			return this;
		};
		obj.inverse = function () {
			this[0] *= -1;
			this[1] *= -1;
			this[2] *= -1;
			return this;
		};
		obj.length = function () {
			var x = this[0], y = this[1], z = this[2], w = this[3];
			return Math.sqrt(x*x + y*y + z*z + w*w);
		};
		obj.normalize = function () {
			var x = this[0], y = this[1], z = this[2], w = this[3],
			    len = Math.sqrt(x*x + y*y + z*z + w*w);
			if(len === 0) {
				this[0] = this[1] = this[2] = this[3] = 0;
			} else {
				len = 1/len;
				this[0] = x * len;
				this[1] = y * len;
				this[2] = z * len;
				this[3] = w * len;
			}
			return this;
		};
		obj.multiply = function (quat) {
			var qax = this[0], qay = this[1], qaz = this[2], qaw = this[3],
			    qbx = quat[0], qby = quat[1], qbz = quat[2], qbw = quat[3];
			this[0] = qax*qbw + qaw*qbx + qay*qbz - qaz*qby;
			this[1] = qay*qbw + qaw*qby + qaz*qbx - qax*qbz;
			this[2] = qaz*qbw + qaw*qbz + qax*qby - qay*qbx;
			this[3] = qaw*qbw - qax*qbx - qay*qby - qaz*qbz;
			return this;
		};
		obj.multiplyVec3 = function (vec) {
			var  x =  vec[0],  y =  vec[1],  z =  vec[2],
			    qx = this[0], qy = this[1], qz = this[2], qw = this[3],
			    ix = qw*x + qy*z - qz*y,
			    iy = qw*y + qz*x - qx*z,
			    iz = qw*z + qx*y - qy*x,
			    iw = -qx*x - qy*y - qz*z;
			vec[0] = ix*qw + iw*-qx + iy*-qz - iz*-qy;
			vec[1] = iy*qw + iw*-qy + iz*-qx - ix*-qz;
			vec[2] = iz*qw + iw*-qz + ix*-qy - iy*-qx;
			return this;
		};
		obj.toMat3 = function () {
			var x = this[0], y = this[1], z = this[2], w = this[3],
			    x2 = x + x, y2 = y + y, z2 = z + z,
			    xx = x*x2, xy = x*y2, xz = x*z2,
			    yy = y*y2, yz = y*z2, zz = z*z2,
			    wx = w*x2, wy = w*y2, wz = w*z2;
			return [
				1 - (yy + zz),
				xy - wz,
				xz + wy,
				xy + wz,
				1 - (xx + zz),
				yz - wx,
				xz - wy,
				yz + wx,
				1 - (xx + yy)
			];
		};
		obj.toMat4 = function (quat) {
			var x = this[0], y = this[1], z = this[2], w = this[3],
			    x2 = x + x, y2 = y + y, z2 = z + z,
			    xx = x*x2, xy = x*y2, xz = x*z2,
			    yy = y*y2, yz = y*z2, zz = z*z2,
			    wx = w*x2, wy = w*y2, wz = w*z2;
			return [
				1 - (yy + zz),
				xy - wz,
				xz + wy,
				0,
				xy + wz,
				1 - (xx + zz),
				yz - wx,
				0,
				xz - wy,
				yz + wx,
				1 - (xx + yy),
				0,
				0,
				0,
				0,
				1
			];
		};
		obj.slerp = function (quat, slerp) {
			var cosHalfTheta = 
				this[0]*this[0] + this[1]*this[1] + this[2]*this[2] + this[3]*this[3];
			if (Math.abs(cosHalfTheta) >= 1.0){
				return this;
			}
			var halfTheta = Math.acos(cosHalfTheta),
			    sinHalfTheta = Math.sqrt(1.0 - cosHalfTheta*cosHalfTheta);
			if (Math.abs(sinHalfTheta) < 0.001){
				this[0] = (this[0]*0.5 + quat[0]*0.5);
				this[1] = (this[1]*0.5 + quat[1]*0.5);
				this[2] = (this[2]*0.5 + quat[2]*0.5);
				this[3] = (this[3]*0.5 + quat[3]*0.5);
				return this;
			}
			var ratioA = Math.sin((1 - slerp)*halfTheta) / sinHalfTheta,
			    ratioB = Math.sin(slerp*halfTheta) / sinHalfTheta;
			this[0] = (this[0]*ratioA + quat[0]*ratioB);
			this[1] = (this[1]*ratioA + quat[1]*ratioB);
			this[2] = (this[2]*ratioA + quat[2]*ratioB);
			this[3] = (this[3]*ratioA + quat[3]*ratioB);
			return this;
		};
		obj.toString = obj.str = function () {
			return this.toArray().join(',');
		};
		obj.toArray = function () {
			return [
				this[0], this[1],
				this[2], this[3]
			];
		};
		obj.toFloat32Array = function () {
			return new Float32Array(this.toArray());
		};
		obj.toFloat64Array = function () {
			return new Float64Array(this.toArray());
		};
		return obj;
	};
	(module.exports = Quaternion).create(quaternion);
})();
});
(function(){"use strict";
	/*global module name api*/
	if (typeof module !== 'undefined') {
		module.exports = api.freeze();
	} else {
		api.freeze("glMath", this);
	}
}).call(this);
}).call(this);
});
api.provide("/base/util", function (require, module, exports) {
/*global module client*/
(function(){"use strict";
	var util = module.exports = client.require('util');
	if (!util) throw 'client did not provide util module.';
}).call(this);
});
api.provide("/base/WebGL", function (require, module, exports) {
/*global module require client window Image*/
(function(){"use strict";
	var util = require('util'),
	    Emitter = client.require('core').require('Emitter');
	function WebGL (canvas, opts) {return this.initWebGL(canvas, opts);}
	var webGL = WebGL.prototype;
	util.inherits(WebGL, Emitter);
	WebGL.check = function (obj) {
		return Emitter.check(obj) && obj.initWebGL === webGL.initWebGL;
	};
	var getWebGLContext = (function(){
		var names = ["webgl", "experimental-webgl", "webkit-3d", "moz-webgl"],
		    tick = 1000 / 60,
		    len = names.length,
		    win = window;
		win.requestAnimFrame =
			win.requestAnimationFrame ||
			win.webkitRequestAnimationFrame ||
			win.mozRequestAnimationFrame ||
			win.oRequestAnimationFrame ||
			win.msRequestAnimationFrame ||
			function (cb) {
				win.setTimeout(cb, tick);
			};
		return function (canvas, opts) {
			var context, i = 0;
			for (; i < len; i += 1) {
				try {
					context = canvas.getContext(names[i], opts);
				} catch(e) {}
				if (context) break;
			}
			return context;
		};
	}).call(this);
	WebGL.create = function (obj, args) {
		obj = obj || {};
		if (!Emitter.check(obj)) {
			Emitter.create(obj);
		}
		obj.initWebGL = webGL.initWebGL || function (canvas, opts) {
			this.gl = getWebGLContext(canvas, opts);
			return this.initEmitter();
		};
		obj.createVertexShader = function (src) {
			var gl = this.gl,
			    shdr = gl.createShader(gl.VERTEX_SHADER);
			return this.compileShader(shdr, src, 'vertex');
		};
		obj.createFragmentShader = function (src) {
			var gl = this.gl,
			    shdr = gl.createShader(gl.FRAGMENT_SHADER);
			return this.compileShader(shdr, src, 'fragment');
		};
		obj.compileShader = function (shdr, src, type) {
			if (typeof src !== 'string' && src && src.length && src.join) {
				src = src.join('\n');
			}
			var gl = this.gl,
			    err;
			gl.shaderSource(shdr, src);
			gl.compileShader(shdr);
			if (!gl.getShaderParameter(shdr, gl.COMPILE_STATUS)) {
				err = gl.getShaderInfoLog(shdr);
				gl.deleteShader(shdr);
				throw err;
			}
			this.emit('newShader', shdr, type);
			return shdr;
		};
		obj.createProgram = function (vShdr, fShdr) {
			var gl = this.gl,
			    prgm = gl.createProgram(),
			    err;
			gl.attachShader(prgm, vShdr);
			gl.attachShader(prgm, fShdr);
			gl.linkProgram(prgm);
			if (!gl.getProgramParameter(prgm, gl.LINK_STATUS)) {
				err = gl.getProgramInfoLog(prgm);
				gl.deleteProgram(prgm);
				throw err;
			}
			this.emit('newProgram', prgm);
			return prgm;
		};
		obj.createArrayBuffer = function (data, cleanup, usage) {
			var gl = this.gl;
			return this.createBufferData(
				gl.ARRAY_BUFFER, data, usage || gl.STATIC_DRAW, cleanup);
		};
		obj.createElementArrayBuffer = function (data, cleanup, usage) {
			var gl = this.gl;
			return this.createBufferData(
				gl.ELEMENT_ARRAY_BUFFER, data, usage || gl.STATIC_DRAW, cleanup);
		};
		obj.createBufferData = function (targ, data, usage, cleanup) {
			var gl = this.gl,
			    buff = gl.createBuffer();
			gl.bindBuffer(targ, buff);
			gl.bufferData(targ, data, usage);
			if (cleanup) gl.bindBuffer(targ, null);
			this.emit('newBuffer', buff, targ, data, usage);
			return buff;
		};
		obj.extractProgramAttrs = function (prgm, obj, attrs) {
			obj = obj || {};
			var gl = this.gl,
			    len = attrs.length, i = 0,
			    attr;
			for (; i < len; i += 1) {
				attr = obj[attrs[i]] = gl.getAttribLocation(prgm, attrs[i]);
			}
			return obj;
		};
		obj.extractProgramUniforms = function (prgm, obj, uniforms) {
			obj = obj || {};
			var gl = this.gl,
			    len = uniforms.length, i = 0;
			for (; i < len; i += 1) {
				obj[uniforms[i]] = gl.getUniformLocation(prgm, uniforms[i]);
			}
			return obj;
		};
		obj.loadTexture = function (url, cb) {
			var gl = this.gl,
			    texture = gl.createTexture(),
			    image = texture.image = new Image();
			image.onload = function () {
				gl.bindTexture(gl.TEXTURE_2D, texture);
				gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, image);
				gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
				gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR);//_MIPMAP_LINEAR);
				gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
				gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
				gl.bindTexture(gl.TEXTURE_2D, null);
				util.safeApply(cb, this, [texture, image]);
			};
			image.src = url;
			return texture;
		};
		if (args) obj.initWebGL.apply(obj, args);
		return obj;
	};
	(module.exports = WebGL).create(webGL);
}).call(this);
});
api.provide("/base/Obj3d", function (require, module, exports) {
/*global module require client*/
(function(){"use strict";
	var util = require('util'),
	    Emitter = client.require('core').require('Emitter');
	function Obj3d (parent) {return this.initObj3d(parent);}
	util.inherits(Obj3d, Emitter);
	var obj3d = Obj3d.prototype;
	Obj3d.check = function (obj) {
		return Emitter.check(obj) && obj.initObj3d === obj3d.initObj3d;
	};
	Obj3d.create = function (obj) {
		obj = obj || {};
		if (!Emitter.check(obj)) {
			Emitter.create(obj);
		}
		obj.initObj3d = obj3d.initObj3d || function (parent) {
			this.world = this;
			this.children = [];
			if (parent) parent.add(this);
			return this;
		};
		obj.uid = function (refresh) {
			var charCode = util.randomCharCode;
			if (refresh) delete this._uid;
			return (this._uid = this._uid || String.fromCharCode(
				charCode(), charCode(), charCode(), charCode()
			) + (+new Date()));
		};
		obj.add = function (obj) {
			if (!Obj3d.check(obj)) return this;
			if (obj !== this) {
				if (obj.parent) {
					if (obj.parent === this) {
						return this;
					}
					obj.parent.remove(obj);
				}
				obj.index = this.children.length;
				this.children.push(obj);
				obj.parent = this;
			}
			obj.world = this.world || this;
			return this;
		};
		obj.remove = function (obj) {
			var index;
			if (!obj || obj.parent !== this) return this;
			if (obj.index && this.children[obj.index] === obj) {
				index = obj.index;
			} else {
				index = this.children.indexOf(obj);
			}
			this.children.splice(index, 1);
			delete obj.index;
			delete obj.parent;
			delete obj.world;
			this.children.forEach(function (obj, ind) {
				if (ind >= index) {
					obj.index = ind;
				}
			});
			return this;
		};
		obj.enumerate = function (fn) {
			this.children.forEach(fn, this);
			return this;
		};
		obj.enumAsync = function (fn, cb) {
			var that = this,
			    count = this.children.length;
			function end () {
				count -= 1;
				if (count === 0) {
					cb.call(that, arguments);
				}
			}
			return this.children.forEach(function (val, ind, obj) {
				fn.call(this, end, val, ind, obj);
			}, this);
		};
		obj.enumInvoke = function (name, args) {
			this.children.forEach(function (obj) {
				obj[name].apply(obj, args);
			});
			return this;
		};
		obj.enumInvokeAsync = function (name, cb, args) {
			args = args || [];
			args.unshift(null);
			return this.enumAsync(function (end, obj) {
				args[0] = end;
				obj[name].apply(obj, args);
			}, cb);
		};
		obj.load = function (cb) {
			util.safeApply(cb, this);
			return this;
		};
		obj.buffer = function () {
			return this;
		};
		obj.render = function () {
			return this;
		};
		obj.update = function () {
			return this;
		};
		return obj;
	};
	(module.exports = Obj3d).create(obj3d);
}).call(this);
});
api.provide("/behavior/Shaded", function (require, module, exports) {
/*global module*/
(function(){"use strict";
	function Shaded (material) {return this.initShaded(material);}
	var shaded = Shaded.prototype;
	Shaded.check = function (obj) {
		return obj && obj.initShaded === shaded.initShaded;
	};
	Shaded.create = function (obj, args) {
		obj = obj || {};
		obj.initShaded = shaded.initShaded || function (material) {
			this.material = material;
			this.shader = material.shader;
			return this;
		};
		obj.renderShaded = function () {
			this.shader.use(this.transformation);
			return this;
		};
		if (args) obj.initShaded.apply(obj, args);
		return obj;
	};
	(module.exports = Shaded).create(shaded);
}).call(this);
});
api.provide("/behavior/Movable", function (require, module, exports) {
/*global module require*/
(function(){"use strict";
	function Movable () {return this.initMovable();}
	var util = require('../base/util'),
	    glMath = require('../glMath'),
	    movable = Movable.prototype,
	    Vector3 = glMath.require('Vector3'),
	    Matrix4 = glMath.require('Matrix4'),
	    Quaternion = glMath.require('Quaternion');
	Movable.check = function (obj) {
		return obj && obj.initMovable === movable.initMovable;
	};
	Movable.create = function (obj, args) {
		obj = obj || {};
		obj.initMovable = movable.initMovable || function () {
			this.transformation = new Matrix4().identity();
			this.rotation = new Vector3([0.0, 0.0, 0.0]);
			this.position = new Vector3([0.0, 0.0, 0.0]);
			this.scale = new Vector3([1.0, 1.0, 1.0]);
			return this;
		};
		obj.transform = function (fn) {
			util.safeApply(fn, this);
			return this.update();
		};
		obj.updateMoveable = function (parentTrans) {
			this.syncTransformation(parentTrans);
			return this.enumerate(function (obj) {
				if (Movable.check(obj)) {
					obj.update(this.transformation);
				}
			});
		};
		obj.syncTransformation = function (parentTrans) {
			if (parentTrans) {
				this.tranformation.set(parentTrans);
			} else {
				this.transformation.identity();
			}
			this.transformation.
				scale(this.scale).
				rotateX(glMath.degToRad(this.rotation[0])).
				rotateY(glMath.degToRad(this.rotation[1])).
				rotateZ(glMath.degToRad(this.rotation[2])).
				translate(this.position);
			return this;
		};
		obj.renderMovable = obj.syncTransformation;
		if (args) obj.init.apply(obj, args);
		return obj;
	};
	(module.exports = Movable).create(movable);
}).call(this);

});
api.provide("/behavior/Colored", function (require, module, exports) {
/*global module require*/
(function(){"use strict";
	function Colored () {return this.initColored();}
	var Vector4 = require('/glMath').require('Vector4'),
	    colored = Colored.prototype;
	Colored.check = function (obj) {
		return obj && obj.initColored === colored.initColored;
	};
	Colored.create = function (obj, args) {
		obj = obj || {};
		obj.initColored = colored.initColored || function () {
			this.hues = [new Vector4([1.0, 1.0, 1.0, 1.0])];
			return this;
		};
		if (args) obj.initColored.apply(obj, args);
		return obj;
	};
	(module.exports = Colored).create(colored);
}).call(this);
});
api.provide("/behavior/Boundary", function (require, module, exports) {
/*global module*/
(function(){"use strict";
	function Boundary () {return this.initBoundary();}
	var boundary = Boundary.prototype;
	Boundary.check = function (obj) {
		return obj && obj.initBoundary === boundary.initBoundary;
	};
	Boundary.create = function (obj, args) {
		obj = obj || {};
		obj.initBoundary = boundary.initBoundary || function () {
			this.boundaryType = Boundary.Cuboid;
			return this;
		};
		if (args) obj.initBoundary.apply(obj, args);
		return obj;
	};
	Boundary.typeNames = ['Cuboid', 'Ellipsoid'];
	Boundary.Cuboid = 0;
	Boundary.Ellipsoid = 1;
	(module.exports = Boundary).create(boundary);
}).call(this);
});
api.provide("/behavior/Animated", function (require, module, exports) {
/*global module*/
(function(){"use strict";
	function Animated () {return this.initAnimated();}
	var animated = Animated.prototype;
	Animated.check = function (obj) {
		return obj && obj.initAnimated === animated.initAnimated;
	};
	Animated.create = function (obj, args) {
		obj = obj || {};
		obj.initAnimated = animated.initAnimated || function () {
			return this;
		};
		if (args) obj.initAnimated.apply(obj, args);
		return obj;
	};
	(module.exports = Animated).create(animated);
}).call(this);
});
api.provide("/behavior/Illuminated", function (require, module, exports) {
/*global module*/
(function(){"use strict";
	function Illuminated () {return this.initIlluminated();}
	var illuminated = Illuminated.prototype;
	Illuminated.check = function (obj) {
		return obj && obj.initIlluminated === illuminated.initIlluminated;
	};
	Illuminated.create = function (obj, args) {
		obj = obj || {};
		obj.initIlluminated = illuminated.initIlluminated || function (parent) {
			return this;
		};
		if (args) obj.initIlluminated.apply(obj, args);
		return obj;
	};
	(module.exports = Illuminated).create(illuminated);
}).call(this);
});
api.provide("/behavior/PolygonMesh", function (require, module, exports) {
/*global module require*/
(function(){"use strict";
	var util = require('../base/util');
	function PolygonMesh (json) {return this.initPolygonMesh(json);}
	function NodeList (owner) {return NodeList.create(null, [owner]);}
	function Triangle () {return this.initPolygonMeshPolygon();}
	function Surface () {return this.initPolygonMeshSurface();}
	function Polygon () {return this.initPolygonMeshPolygon();}
	function Vertex () {return this.initPolygonMeshVertex();}
	function Group () {return this.initPolygonMeshGroup();}
	function Edge () {return this.initPolygonMeshEdge();}
	function Node () {return this.initPolygonMeshNode();}
	util.inherits(Triangle, Node);
	util.inherits(Surface, Node);
	util.inherits(Polygon, Node);
	util.inherits(Vertex, Node);
	util.inherits(Group, Node);
	util.inherits(Edge, Node);
	var Vector3 = require('/glMath').require('Vector3'),
	    polygonMesh = PolygonMesh.prototype,
	    nodeList = NodeList.prototype,
	    triangle = Triangle.prototype,
	    surface = Surface.prototype,
	    polygon = Polygon.prototype,
	    vertex = Vertex.prototype,
	    group = Group.prototype,
	    edge = Edge.prototype,
	    node = node.prototype;
	PolygonMesh.check = function (obj) {
		return obj && obj.initPolygonMesh === polygonMesh.initPolygonMesh;
	};
	NodeList.check = function (obj) {
		return obj && obj.initPolygonMeshNodeList === nodeList.initPolygonMeshNodeList;
	};
	Triangle.check = function (obj) {
		return Node.check(obj) && obj.initPolygonMeshTriangle === triangle.initPolygonMeshTriangle;
	};
	Surface.check = function (obj) {
		return Node.check(obj) && obj.initPolygonMeshSurface === surface.initPolygonMeshSurface;
	};
	Polygon.check = function (obj) {
		return Node.check(obj) && obj.initPolygonMeshPolygon === polygon.initPolygonMeshPolygon;
	};
	Vertex.check = function (obj) {
		return Node.check(obj) && obj.initPolygonMeshVertex === vertex.initPolygonMeshVertex;
	};
	Group.check = function (obj) {
		return Node.check(obj) && obj.initPolygonMeshGroup === group.initPolygonMeshGroup;
	};
	Edge.check = function (obj) {
		return Node.check(obj) && obj.initPolygonMeshEdge === edge.initPolygonMeshEdge;
	};
	Node.check = function (obj) {
		return obj && obj.initPolygonMeshNode === node.initPolygonMeshNode;
	};
	PolygonMesh.create = function (obj, args) {
		obj = obj || {};
		obj.initPolygonMesh = polygonMesh.initPolygonMesh || function (json) {
			this.meshVertices = new NodeList(this, Vertex.check);
			this.meshSurfaces = new NodeList(this, Surface.check);
			return this.polygonMeshFromJSON(json);
		};
		obj.polygonMeshFromJSON = function (json) {
			if (!json) return this;
			if (typeof json === 'string') {
				json = JSON.parse(json);
			}
			this.meshVertices.set(json.vertices, function (vertex) {
				return new Vertex(this).fromJSON(vertex);
			});
			this.meshSurfaces.set(json.polygons, function (surface) {
				return new Surface(this).fromJSON(surface);
			});
			return this.computePolygonMesh();
		};
		obj.polygonMeshToJSON = function () {
			return JSON.stringify({
				vertices: this.meshVertices.toJSON(),
				surfaces: this.meshSurfaces.toJSON()
			});
		};
		obj.emptyPolygonMesh = function () {
			this.meshVertices.set([]);
			this.meshSurfaces.set([]);
			return this;
		};
		obj.computePolygonMesh = function () {
			return this;
		};
		obj.bufferPolygonMesh = function () {
			return this;
		};
		obj.renderPolygonMesh = function () {
			return this;
		};
		if (args) obj.initPolygonMesh.apply(obj, args);
		return obj;
	};
	NodeList.create = function (obj, args) {
		obj = obj || [];
		obj.initPolygonMeshNodeList = nodeList.initPolygonMeshNodeList || function (owner, check) {
			this.owner = this;
			this.check = check || Node.check;
			return this;
		};
		obj.enumerate = function (fn) {
			return this.forEach(fn, this);
		};
		obj.find = function (child) {
			if (!this.check(child)) return -1;
			return this.indexOf(child);
		};
		obj.add = function (child) {
			if (!~this.find(child)) this.push(child);
			return this;
		};
		obj.remove = function (child)  {
			var index = this.find(child);
			if (~index) this.splice(index, 1);
			return this;
		};
		obj.set = function (list, fn) {
			while (this[0]) this.pop();
			if (list && list.forEach) {
				if (typeof fn !== 'function') {
					fn = function (val) {
						return val;
					};
				}
				list.forEach(function (val, ind, obj) {
					this[ind] = fn.call(this, val, ind, obj);
				}, this);
			}
			return this;
		};
		obj.toJSON = function () {
			var list = [];
			this.enumerate(function (node) {
				var json = node && node.toJSON && node.toJSON();
				if (json) list.push(json);
			});
			return '[' + list.join(',') + ']';
		};
		if (args) obj.initNodeList.apply(obj, args);
		return obj;
	};
	Triangle.create = function (obj, args) {
		obj = obj || {};
		if (!Node.check(obj)) {
			Node.create(obj);
		}
		obj.initPolygonMeshTriangle = triangle.initPolygonMeshTriangle || function (polygon) {
			this.adjacent = new NodeList(this, Triangle.check);
			this.vertices = new NodeList(this, Vertex.check);
			return this.initPolygonMeshNode(polygon);
		};
		obj.compute = function () {
			return this;
		};
		obj.fromJSON = function (json) {
			if (!json) return this;
			if (typeof json === 'string') {
				json = JSON.parse(json);
			}
			var that = this;
			this.vertices.set(json.vertices, function (vertex) {
				return new Vertex().fromJSON(vertex).addTo(that);
			});
			return this;
		};
		obj.toJSON = function () {
			return JSON.stringify({
				vertices: this.vertices.toJSON()
			});
		};
		if (args) obj.initTriangle.apply(obj, args);
		return obj;
	};
	Surface.create = function (obj, args) {
		obj = obj || {};
		if (!Node.check(obj)) {
			Node.create(obj);
		}
		obj.initPolygonMeshSurface = surface.initPolygonMeshSurface || function (mesh) {
			this.adjacent = new NodeList(this, Surface.check);
			this.polygons = new NodeList(this, Polygon.check);
			this.groups = new NodeList(this, Group.check);
			return this.initPolygonMeshNode(mesh);
		};
		obj.compute = function () {
			return this;
		};
		obj.fromJSON = function (json) {
			if (!json) return this;
			if (typeof json === 'string') {
				json = JSON.parse(json);
			}
			var that = this;
			this.polygons.set(json.polygons, function (polygon) {
				return new Polygon(this).fromJSON(polygon).addTo(that);
			});
			this.groups.set(json.groups, function (group) {
				return new Group(this).fromJSON(group).addTo(that);
			});
			return this;
		};
		obj.toJSON = function () {
			return JSON.stringify({
				polygons: this.polygons.toJSON(),
				groups: this.groups.toJSON()
			});
		};
		if (args) obj.initSurface.apply(obj, args);
		return obj;
	};
	Polygon.create = function (obj, args) {
		obj = obj || {};
		if (!Node.check(obj)) {
			Node.create(obj);
		}
		obj.initPolygonMeshPolygon = polygon.initPolygonMeshPolygon || function (surface) {
			this.triangles = new NodeList(this, Triangle.check);
			this.adjacent = new NodeList(this, Polygon.check);
			this.edges = new NodeList(this, Edge.check);
			return this.initPolygonMeshNode(surface);
		};
		obj.compute = function () {
			return this;
		};
		obj.fromJSON = function (json) {
			if (!json) return this;
			if (typeof json === 'string') {
				json = JSON.parse(json);
			}
			var that = this;
			this.edges.set(json.edges, function (edge) {
				return new Edge(this).fromJSON(edge).addTo(that);
			});
			return this;
		};
		obj.toJSON = function () {
			return JSON.stringify({
				edges: this.edges.toJSON()
			});
		};
		if (args) obj.initPolygon.apply(obj, args);
		return obj;
	};
	Vertex.create = function (obj, args) {
		obj = obj || {};
		if (!Node.check(obj)) {
			Node.create(obj);
		}
		obj.initPolygonMeshVertex = vertex.initPolygonMeshVertex || function (edge) {
			this.triangles = new NodeList(this, Triangle.check);
			this.adjacent = new NodeList(this, Vertex.check);
			this.surfaces = new NodeList(this, Surface.check);
			this.polygons = new NodeList(this, Polygon.check);
			this.edges = new NodeList(this, Edge.check);
			this.vec = new Vector3();
			return this.initPolygonMeshNode(edge);
		};
		obj.compute = function () {
			return this;
		};
		obj.fromJSON = function (json) {
			if (!json) return this;
			if (typeof json === 'string') {
				json = JSON.parse(json);
			}
			return this.vec.set(json);
		};
		obj.toJSON = function () {
			return '[' + this.vec.str() + ']';
		};
		if (args) obj.initVertex.apply(obj, args);
		return obj;
	};
	Group.create = function (obj, args) {
		obj = obj || {};
		obj.initPolygonMeshGroup = group.initPolygonMeshGroup || function (surface) {
			return this.initPolygonMeshNode(surface);
		};
		obj.compute = function () {
			return this;
		};
		obj.fromJSON = function (json) {
			return this;
		};
		obj.toJSON = function () {
			var json = '';
			return json;
		};
		if (args) obj.initGroup.apply(obj, args);
		return obj;
	};
	Edge.create = function (obj, args) {
		obj = obj || {};
		if (!Node.check(obj)) {
			Node.create(obj);
		}
		obj.initPolygonMeshEdge = edge.initPolygonMeshEdge || function (triangle) {
			this.adjacent = new NodeList(this, Edge.check);
			this.polygons = new NodeList(this, Polygon.check);
			this.vertices = new NodeList(this, Vertex.check);
			this.edges = new NodeList(this, Edge.check);
			return this.initPolygonMeshNode(triangle);
		};
		obj.compute = function () {
			return this;
		};
		obj.fromJSON = function (json) {
			if (!json) return this;
			if (typeof json === 'string') {
				json = JSON.parse(json);
			}
			var that = this;
			this.vertices.set(json.vertices, function (vertex) {
				return new Vertex().fromJSON(vertex).addTo(that);
			});
			return this;
		};
		obj.toJSON = function () {
			return JSON.stringify({
				vertices: this.vertices.toJSON()
			});
		};
		if (args) obj.initEdge.apply(obj, args);
		return obj;
	};
	Node.create = function (obj, args) {
		obj = obj || {};
		obj.initPolygonMeshNode = node.initPolygonMeshNode || function () {
			this.parents = new NodeList(this);
			this.children = new NodeList(this);
			return this;
		};
		obj.addTo = function (parent) {
			if (Node.check(parent)) {
				parent.children.add(this);
				this.parents.add(parent);
			}
			return this;
		};
		obj.add = function (child) {
			if (Node.check(child)) {
				this.children.add(child);
				child.parents.add(this);
			}
			return this;
		};
		obj.remove = function (child) {
			if (Node.check(child)) {
				this.children.remove(child);
				child.parents.remove(this);
			}
			return this;
		};
		if (args) obj.initNode.apply(obj, args);
		return obj;
	};
	(module.exports = PolygonMesh).create(polygonMesh);
	(module.exports = NodeList).create(nodeList);
	(module.exports = Triangle).create(triangle);
	(module.exports = Surface).create(surface);
	(module.exports = Polygon).create(polygon);
	(module.exports = Vertex).create(vertex);
	(module.exports = Group).create(group);
	(module.exports = Edge).create(edge);
	(module.exports = Node).create(node);
}).call(this);
});
api.provide("/behavior/ParticleField", function (require, module, exports) {
/*global module*/
(function(){"use strict";
	function ParticleField () {return this.initParticleField();}
	var particleField = ParticleField.prototype;
	ParticleField.check = function (obj) {
		return obj && obj.initParticleField === particleField.initParticleField;
	};
	ParticleField.create = function (obj, args) {
		obj = obj || {};
		obj.initParticleField = particleField.initParticleField || function (parent) {
			return this;
		};
		if (args) obj.init.apply(obj, args);
		return obj;
	};
	(module.exports = ParticleField).create(particleField);
}).call(this);
});
api.provide("/scene/Camera", function (require, module, exports) {
/*global module require Float32Array Float64Array*/
(function(){"use strict";
	var util = require('../base/util'),
	    glMath = require('../glMath'),
	    Vector3 = glMath.require('Vector3'),
	    Matrix3 = glMath.require('Matrix3'),
	    Matrix4 = glMath.require('Matrix4'),
	    Obj3d = require('../base/Obj3d'),
	    Movable = require('../behavior/Movable'),
	    Animated = require('../behavior/Animated');
	glMath = glMath.require('utils');
	function Camera (world) {return this.initCamera(world);}
	util.inherits(Camera, Obj3d);
	var camera = Camera.prototype;
	Camera.check = function (obj) {
		return Obj3d.check(obj) && obj.initCamera === camera.initCamera;
	};
	Camera.create = function (obj, args) {
		obj = obj || {};
		if (!Obj3d.check(obj)) {
			Obj3d.create(obj);
		}
		obj.initCamera = camera.initCamera || function (world) {
			if (!world) return this;
			this.initObj3d(world);
			this.mvpNormal = Matrix4.create(new Float32Array(16));
			this.mvp = Matrix4.create(new Float32Array(16));
			this.view = new Matrix4();
			this.projection = new Matrix4();
			this.target = new Vector3([0.0, 0.0, 0.0]);
			this.up = new Vector3([0.0, 1.0, 0.0]);
			this.eye = new Vector3([0.0, 2.0, 10.0]);
			this.far = 1000.0;
			this.near = 0.001;
			this.fovy = 45.0;
			this.aspect = 0.75;
			return this;
		};
		obj.projectLookAt = function () {
			this.view.lookAt(this.eye, this.target, this.up);
			this.projection.perspective(this.fovy, this.aspect, this.near, this.far);
			return this;
		};
		obj.flatten = function (model) {
			var world = this.world,
			    attrs = world.shader.attrs,
			    mvpNormal = this.mvpNormal;
			this.mvp.set(this.projection).multiply(this.view).multiply(model);
			mvpNormal.set(this.view).multiply(model).inverse().transpose();
			world.gl.uniformMatrix4fv(attrs.mvp, false, this.mvp);
			world.gl.uniformMatrix4fv(attrs.mvpNormal, false, mvpNormal);
			return this;
		};
		obj.moveForward = function (scale) {
			var dir = this.target.copy().
				direction(this.eye).
				scale(scale);
			this.target.add(dir);
			this.eye.add(dir);
			return this;
		};
		obj.moveUp = function (scale) {
			var dir, right = this.target.copy().
				direction(this.eye).
				cross(this.up);
			dir = this.target.copy().
				direction(this.eye).
				cross(right).
				negate().
				scale(scale);
			this.target.add(dir);
			this.eye.add(dir);
			return this;
		};
		obj.moveRight = function (scale) {
			var dir = this.target.copy().
				direction(this.eye).
				cross(this.up).
				scale(scale);
			this.target.add(dir);
			this.eye.add(dir);
			return this;
		};
		obj.orbitPitch = function (deg) {
			var dir = this.target.copy().
				direction(this.eye).
				cross(this.up);
			new Matrix4().identity().
				rotate(glMath.degToRad(deg), dir).
				multiplyVec3(this.eye).
				multiplyVec3(this.up);
			return this;
		};
		obj.orbitRoll = function (deg) {
			var dir = this.target.copy().
				direction(this.eye);
			new Matrix4().identity().
				rotate(glMath.degToRad(deg), dir).
				multiplyVec3(this.eye).
				multiplyVec3(this.up);
			return this;
		};
		obj.orbitYaw = function (deg) {
			var dir, right = this.target.copy().
				direction(this.eye).
				cross(this.up);
			dir = this.target.copy().
				direction(this.eye).
				cross(right);
			new Matrix4().identity().
				rotate(glMath.degToRad(deg), dir).
				multiplyVec3(this.eye).
				multiplyVec3(this.up);
			return this;
		};
		obj.lookOrbitPitch = function (deg) {
			return this;
		};
		obj.lookOrbitYaw = function (deg) {
			return this;
		};
		if (args) obj.initCamera.apply(obj, args);
		return obj;
	};
	(module.exports = Camera).create(camera);
}).call(this);
});
api.provide("/scene/World", function (require, module, exports) {
/*global module require*/
(function(){"use strict";
	var util = require('../base/util'),
	    glMath = require('../glMath'),
	    Vector3 = glMath.require('Vector3'),
	    Vector4 = glMath.require('Vector4'),
	    Obj3d = require('../base/Obj3d'),
	    Camera = require('../scene/Camera');
	function World (app) {return this.initWorld(app);}
	util.inherits(World, Obj3d);
	var world = World.prototype;
	World.check = function (obj) {
		return Obj3d.check(obj) && obj.initWorld === world.initWorld;
	};
	World.create = function (obj, args) {
		obj = obj || {};
		if (!Obj3d.check(obj)) {
			Obj3d.create(obj);
		}
		obj.initWorld = world.initWorld || function (app) {
			this.gl = app.webgl.gl;
			this.app = app;
			this.webgl = app.webgl;
			this.initObj3d(this);
			this.camera = new Camera(this);
			this.lightDir = new Vector3([0.0, 0.5, 0.5]);
			this.sampler2d = 0;
			this.clearColor = new Vector4([0.0, 0.0, 0.0, 1.0]);
			this.clearDepth = 1.0;
			return this;
		};
		obj.load = function (cb) {
			return this.enumInvokeAsync('load', cb, [this.webgl, this.gl]);
		};
		obj.buffer = function () {
			var gl = this.gl,
			    c = this.clearColor;
			gl.clearColor(c[0], c[1], c[2], c[3]);
			gl.clearDepth(1.0);
			gl.enable(gl.BLEND);
			gl.enable(gl.CULL_FACE);
			gl.enable(gl.DEPTH_TEST);
			gl.enable(gl.TEXTURE_2D);
			this.enumInvoke('buffer', [this.webgl, this.gl]);
			return this;
		};
		obj.reshape = function () {
			var app = this.app;
			this.camera.aspect = app.width / app.height;
			this.gl.viewport(0, 0, app.width, app.height);
			return this;
		};
		obj.render = function () {
			this.clear();
			this.camera.projectLookAt();
			this.enumInvoke('render', [this.webgl, this.gl]);
			this.gl.flush();
			this.gl.finish();
			return this;
		};
		obj.clear = function () {
			this.gl.clear(this.gl.COLOR_BUFFER_BIT | this.gl.DEPTH_BUFFER_BIT);
			return this;
		};
		obj.pick = function (x, y) {
			return this;
		};
		if (args) obj.initWorld.apply(obj, args);
		return obj;
	};
	(module.exports = World).create(world);
}).call(this);
});
api.provide("/scene/Shader", function (require, module, exports) {
/*global module require*/
(function(){"use strict";
	var util = require('../base/util');
	function Shader (world) {return this.initShader(world);}
	var shader = Shader.prototype;
	Shader.check = function (obj) {
		return obj && obj.initShader === shader.initShader;
	};
	Shader.create = function (obj, args) {
		obj = obj || {};
		obj.initShader = shader.initShader || function (world, src) {
			this.world = world;
			this.attrs = {};
			return this.build(src).setup();
		};
		obj.build = function (src) {
			if (this.built) return this;
			var webgl = this.world.webgl;
			this.fragment = webgl.createFragmentShader(src.fragment);
			this.vertex = webgl.createVertexShader(src.vertex);
			this.program = webgl.createProgram(this.fragment, this.vertex);
			this.built = true;
			delete this.ready;
			return this;
		};
		obj.rebuild = function (frag, vert) {
			delete this.built;
			return this.build(frag, vert);
		};
		obj.setup = function (cb) {
			if (this.built && !this.ready) {
				util.safeApply(cb, this,
					[this.world.webgl, this.world.gl, this.attrs, this.program]);
				this.ready = true;
			}
			return this;
		};
		obj.resetup = function () {
			delete this.ready;
			return this.setup();
		};
		obj.use = function (modelMatrix) {
			var world = this.world;
			if (this.built && this.ready) {
				if (world.shader === this) {
					world.camera.flatten(modelMatrix);
					return this;
				}
				world.gl.useProgram(this.program);
				world.shader = this;
				world.camera.flatten(modelMatrix);
			}
			return this;
		};
		if (args) obj.initShader.apply(obj, args);
		return obj;
	};
	(module.exports = Shader).create(shader);
}).call(this);
});
api.provide("/scene/Light", function (require, module, exports) {
/*global module require*/
(function(){"use strict";
	var util = require('../base/util'),
	    Obj3d = require('../base/Obj3d'),
	    Movable = require('../behavior/Movable'),
	    Colored = require('../behavior/Colored'),
	    Animated = require('../behavior/Animated'),
	    Illuminated = require('../behavior/Illuminated');
	function Light (parent) {return this.initLight(parent);}
	var light = Light.prototype;
	util.inherits(Light, Obj3d);
	Light.check = function (obj) {
		return Obj3d.check(obj) && obj.initLight === light.initLight;
	};
	Light.create = function (obj, args) {
		obj = obj || {};
		if (!Obj3d.check(obj)) Obj3d.create(obj);
		if (!Movable.check(obj)) Movable.create(obj);
		if (!Animated.check(obj)) Animated.create(obj);
		if (!Colored.check(obj)) Colored.create(obj);
		if (!Illuminated.check(obj)) Illuminated.create(obj);
		obj.initLight = light.initLight || function (parent) {
			this.initObj3d(parent);
			this.initMovable();
			this.initColored();
			return this;
		};
		if (args) obj.initLight.apply(obj, args);
		return obj;
	};
	(module.exports = Light).create(light);
}).call(this);
});
api.provide("/scene/Entity", function (require, module, exports) {
/*global module require*/
(function(){"use strict";
	var util = require('../base/util'),
	    Vector3 = require('../glMath').require('Vector3'),
	    Obj3d = require('../base/Obj3d'),
	    Shaded = require('../behavior/Shaded'),
	    Movable = require('../behavior/Movable'),
	    Colored = require('../behavior/Colored'),
	    Animated = require('../behavior/Animated'),
	    PolygonMesh = require('../behavior/PolygonMesh'),
	    ParticleField = require('../behavior/ParticleField');
	function Entity (world, material) {return this.initEntity(world, material);}
	var entity = Entity.prototype;
	util.inherits(Entity, Obj3d);
	Entity.check = function (obj) {
		return Obj3d.check(obj) && obj.initEntity === entity.initEntity;
	};
	Entity.create = function (obj, args) {
		obj = obj || {};
		function inherit (constr) {
			if (!constr.check(obj)) constr.create(obj);
		}
		inherit(Obj3d);
		inherit(Shaded);
		inherit(Movable);
		inherit(Colored);
		inherit(Animated);
		inherit(PolygonMesh);
		inherit(ParticleField);
		obj.initEntity = entity.initEntitiy || function (parent, material) {
			this.initObj3d(parent);
			this.initShaded(material);
			this.initMovable();
			this.initColored();
			this.initAnimated();
			this.initPolygonMesh();
			this.initParticleField();
			return this;
		};
		obj.update = function () {
			this.updateMoveable();
			return this;
		};
		obj.buffer = function () {
			this.bufferPolygonMesh();
			return this;
		};
		obj.render = function (cb) {
			this.renderMovable();
			this.renderShaded();
			this.renderPolygonMesh();
			util.safeApply(cb, this);
			return this;
		};
		if (args) obj.initEntity.apply(obj, args);
		return obj;
	};
	(module.exports = Entity).create(entity);
}).call(this);
});
api.provide("/scene/Material", function (require, module, exports) {
/*global module*/
(function(){"use strict";
	function Material (shader) {return this.initMaterial(shader);}
	var material = Material.prototype;
	Material.check = function (obj) {
		return obj && obj.initMaterial === material.initMaterial;
	};
	Material.create = function (obj, args) {
		obj = obj || {};
		obj.initMaterial = material.initMaterial || function (shader) {
			this.shader = shader;
			return this;
		};
		if (args) obj.initMaterial.apply(obj, args);
		return obj;
	};
	(module.exports = Material).create(material);
}).call(this);
});
api.provide("/shaders/BasicShader", function (require, module, exports) {
/*global module require*/
(function(){"use strict";
	var util = require('../base/util'),
	    Shader = require('../scene/Shader');
	function BasicShader (world) {return this.initBasicShader(world);}
	util.inherits(BasicShader, Shader);
	var basicShader = BasicShader.prototype;
	BasicShader.check = function (obj) {
		return Shader.check(obj) && obj.initBasicShader === basicShader.initBasicShader;
	};
	BasicShader.create = function (obj, args) {
		var supr = Shader.prototype;
		obj = obj || {};
		if (!Shader.check(obj)) {
			Shader.create(obj);
		}
		obj.initBasicShader = basicShader.initBasicShader || function (world) {
			return this.initShader(world, {
				fragment: [
					'#ifdef GL_ES',
					'precision mediump float;',
					'#endif',
					'uniform sampler2D sampler2d;',
					'varying float myDot;',
					'varying vec4 myColor;',
					'varying vec2 myTexCoord;',
					'void main(void) {',
						'vec2 texCoord = vec2(myTexCoord.s, 1.0 - myTexCoord.t);',
						'vec4 color = texture2D(sampler2d, texCoord) / 2.0;',
						'color += myColor / 2.0;',
						'color /= 0.5;',
						'gl_FragColor = vec4(color.xyz * myDot, color.a);',
					'}'
				],
				vertex: [
					'uniform mat4 mvp;',
					'uniform mat4 mvpNormal;',
					'uniform vec3 lightDir;',
					'attribute vec3 position;',
					'attribute vec2 texCoord;',
					'attribute vec3 normal;',
					'attribute vec4 color;',
					'varying float myDot;',
					'varying vec4 myColor;',
					'varying vec2 myTexCoord;',
					'void main(void) {',
						'gl_Position = mvp * vec4(position, 1.0);',
						'myColor = color;',
						'myTexCoord = texCoord;',
						'vec4 transNormal = mvpNormal * vec4(normal, 1.0);',
						'myDot = max(dot(transNormal.xyz, lightDir), 0.1);',
					'}'
				]
			});
		};
		obj.setup = function () {
			return supr.setup.call(this, function (webgl, gl, attrs, program) {
				webgl.extractProgramUniforms(program, attrs, ['mvp', 'mvpNormal', 'lightDir', 'sampler2d']);
				webgl.extractProgramAttrs(program, attrs, ['position', 'texCoord', 'normal', 'color']);
			});
		};
		obj.use = function (modelMatrix) {
			var gl = this.world.gl,
			    ld = this.world.lightDir,
			    attrs = this.attrs;
			supr.use.call(this, modelMatrix);
			gl.uniform3f(attrs.lightDir, ld[0], ld[1], ld[2]);
			gl.uniform1i(attrs.sampler2d, this.world.sampler2d);
			return this;
		};
		if (args) obj.initBasicShader.apply(obj, args);
		return obj;
	};
	(module.exports = BasicShader).create(basicShader);
}).call(this);
});
api.provide("/materials/BasicMaterial", function (require, module, exports) {
/*global module require*/
(function(){"use strict";
	var util = require('../base/util'),
	    BasicShader = require('../shaders/BasicShader'),
	    Material = require('../scene/Material');
	function BasicMaterial (world) {return this.initBasicMaterial(world);}
	util.inherits(BasicMaterial, Material);
	var basicMaterial = BasicMaterial.prototype;
	BasicMaterial.check = function (obj) {
		return Material.check(obj) && obj.initBasicMaterial === basicMaterial.initBasicMaterial;
	};
	BasicMaterial.create = function (obj) {
		obj = obj || {};
		if (!Material.check(obj)) {
			Material.create(obj);
		}
		obj.initBasicMaterial = basicMaterial.initBasicMaterial || function (world) {
			return this.initMaterial(new BasicShader(world));
		};
		return obj;
	};
	(module.exports = BasicMaterial).create(basicMaterial);
}).call(this);
});
api.provide("/shapes/Cube", function (require, module, exports) {
/*global module require Uint8Array Float32Array*/
(function(){"use strict";
	var util = require('../base/util'),
	    Entity = require('../scene/Entity'),
	    BasicMaterial = require('../materials/BasicMaterial');
	function Cube (world) {return this.initCube(world);}
	util.inherits(Cube, Entity);
	var cube = Cube.prototype;
	Cube.check = function (obj) {
		return Entity.check(obj) && obj.initCube === cube.initCube;
	};
	Cube.create = function (obj, args) {
		var supr = Entity.prototype;
		obj = obj || {};
		if (!Entity.check(obj)) {
			Entity.create(obj);
		}
		obj.initCube = cube.initCube || function (parent, material) {
			material = material || new BasicMaterial(parent.world);
			return this.initEntity(parent, material).allocate();
		};
		obj.allocate = function () {
			return this.
				giveIndices().
				givePoints().
				giveNormals().
				giveColors();
		};
		obj.dellocate = function () {
			delete this.indices;
			delete this.points;
			delete this.normals;
			delete this.colors;
			return this;
		};
		obj.giveIndices = function () {
			// v0-v1-v2-v3 front
			// v0-v3-v4-v5 right
			// v0-v5-v6-v1 top
			// v1-v6-v7-v2 left
			// v7-v4-v3-v2 bottom
			// v4-v7-v6-v5 back
			if (this.indices) return this;
			this.indices = new Uint8Array([
				 0,  1,  2,   0,  2,  3,
				 4,  5,  6,   4,  6,  7,
				 8,  9, 10,   8, 10, 11,
				12, 13, 14,  12, 14, 15,
				16, 17, 18,  16, 18, 19,
				20, 21, 22,  20, 22, 23
			]);
			this.indices.units = this.indices.length;
			this.indices.size = 1;
			return this;
		};
		obj.givePoints = function () {
			if (this.points) return this;
			this.points = new Float32Array([
				 1.0, 1.0, 1.0,  1.0, 1.0,  -1.0, 1.0, 1.0,  0.0, 1.0,  -1.0,-1.0, 1.0,  0.0, 0.0,   1.0,-1.0, 1.0,  1.0, 0.0,
				 1.0, 1.0, 1.0,  0.0, 1.0,   1.0,-1.0, 1.0,  0.0, 0.0,   1.0,-1.0,-1.0,  1.0, 0.0,   1.0, 1.0,-1.0,  1.0, 1.0,
				 1.0, 1.0, 1.0,  1.0, 0.0,   1.0, 1.0,-1.0,  1.0, 1.0,  -1.0, 1.0,-1.0,  0.0, 1.0,  -1.0, 1.0, 1.0,  0.0, 0.0,
				-1.0, 1.0, 1.0,  1.0, 1.0,  -1.0, 1.0,-1.0,  0.0, 1.0,  -1.0,-1.0,-1.0,  0.0, 0.0,  -1.0,-1.0, 1.0,  1.0, 0.0,
				-1.0,-1.0,-1.0,  0.0, 0.0,   1.0,-1.0,-1.0,  1.0, 0.0,   1.0,-1.0, 1.0,  1.0, 1.0,  -1.0,-1.0, 1.0,  0.0, 1.0,
				 1.0,-1.0,-1.0,  0.0, 0.0,  -1.0,-1.0,-1.0,  1.0, 0.0,  -1.0, 1.0,-1.0,  1.0, 1.0,   1.0, 1.0,-1.0,  0.0, 1.0
			]);
			this.points.units = this.points.length / (this.points.size = 5);
			this.points.vertexSize = 3;
			this.points.texCoordSize = 2;
			return this;
		};
		obj.giveNormals = function () {
			if (this.normals) return this;
			this.normals = new Float32Array([
				 0.0, 0.0, 1.0,   0.0, 0.0, 1.0,   0.0, 0.0, 1.0,   0.0, 0.0, 1.0,
				 1.0, 0.0, 0.0,   1.0, 0.0, 0.0,   1.0, 0.0, 0.0,   1.0, 0.0, 0.0,
				 0.0, 1.0, 0.0,   0.0, 1.0, 0.0,   0.0, 1.0, 0.0,   0.0, 1.0, 0.0,
				-1.0, 0.0, 0.0,  -1.0, 0.0, 0.0,  -1.0, 0.0, 0.0,  -1.0, 0.0, 0.0,
				 0.0,-1.0, 0.0,   0.0,-1.0, 0.0,   0.0,-1.0, 0.0,   0.0,-1.0, 0.0,
				 0.0, 0.0,-1.0,   0.0, 0.0,-1.0,   0.0, 0.0,-1.0,   0.0, 0.0,-1.0
			]);
			this.normals.units = this.normals.length / (this.normals.size = 3);
			return this;
		};
		obj.giveColors = function () {
			if (this.colors) return this;
			this.colors = new Float32Array([
				0.0, 0.0, 1.0, 1.0,   0.0, 0.0, 1.0, 1.0,   0.0, 0.0, 1.0, 1.0,   0.0, 0.0, 1.0, 1.0,
				1.0, 0.0, 0.0, 1.0,   1.0, 0.0, 0.0, 1.0,   1.0, 0.0, 0.0, 1.0,   1.0, 0.0, 0.0, 1.0,
				0.0, 1.0, 0.0, 1.0,   0.0, 1.0, 0.0, 1.0,   0.0, 1.0, 0.0, 1.0,   0.0, 1.0, 0.0, 1.0,
				1.0, 1.0, 0.0, 1.0,   1.0, 1.0, 0.0, 1.0,   1.0, 1.0, 0.0, 1.0,   1.0, 1.0, 0.0, 1.0,
				1.0, 0.0, 1.0, 1.0,   1.0, 0.0, 1.0, 1.0,   1.0, 0.0, 1.0, 1.0,   1.0, 0.0, 1.0, 1.0,
				0.0, 1.0, 1.0, 1.0,   0.0, 1.0, 1.0, 1.0,   0.0, 1.0, 1.0, 1.0,   0.0, 1.0, 1.0, 1.0
			]);
			this.colors.units = this.colors.length / (this.colors.size = 4);
			return this;
		};
		obj.load = function (cb) {
			if (this.textureUrl) {
				this.texture = this.world.webgl.loadTexture(this.textureUrl, cb);
			} else {
				util.safeApply(cb, this);
			}
			return this;
		};
		obj.buffer = function () {
			var webgl = this.world.webgl;
			this.unbuffer();
			if (this.indices) this.indexBuf = webgl.createElementArrayBuffer(this.indices, true);
			if (this.points) this.pointsBuf = webgl.createArrayBuffer(this.points, true);
			if (this.normals) this.normalBuf = webgl.createArrayBuffer(this.normals, true);
			if (this.colors) this.colorBuf = webgl.createArrayBuffer(this.colors, true);
			return this;
		};
		obj.unbuffer = function () {
			var gl = this.world.gl;
			if (this.indexBuf) gl.deleteBuffer(this.indexBuf);
			if (this.pointsBuf) gl.deleteBuffer(this.pointsBuf);
			if (this.normalBuf) gl.deleteBuffer(this.normalBuf);
			if (this.colorBuf) gl.deleteBuffer(this.colorBuf);
			return this;
		};
		obj.render = function (webgl, gl) {
			return supr.render.call(this, function () {
				var attrs = this.shader.attrs;

				if (this.pointsBuf) {
					gl.enableVertexAttribArray(attrs.position);
					gl.bindBuffer(gl.ARRAY_BUFFER, this.pointsBuf);
					gl.vertexAttribPointer(attrs.position, this.points.vertexSize, gl.FLOAT, false, 20, 0);
					if (this.texture) {
						gl.enableVertexAttribArray(attrs.texCoord);
						gl.vertexAttribPointer(attrs.texCoord, this.points.texCoordSize, gl.FLOAT, false, 20, 12);
					}
				}

				if (this.normalBuf) {
					gl.enableVertexAttribArray(attrs.normal);
					gl.bindBuffer(gl.ARRAY_BUFFER, this.normalBuf);
					gl.vertexAttribPointer(attrs.normal, this.normals.size, gl.FLOAT, false, 0, 0);
				}

				if (this.colorBuf) {
					gl.enableVertexAttribArray(attrs.color);
					gl.bindBuffer(gl.ARRAY_BUFFER, this.colorBuf);
					gl.vertexAttribPointer(attrs.color, this.colors.size, gl.FLOAT, false, 0, 0);
				}

				gl.bindBuffer(gl.ARRAY_BUFFER, null);

				if (this.pointsBuf && this.texture) {
					gl.bindTexture(gl.TEXTURE_2D, this.texture);
				}

				gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.indexBuf);
				gl.drawElements(gl.TRIANGLES, this.indices.length, gl.UNSIGNED_BYTE, 0);

				gl.bindBuffer(gl.ELEMENT_ARRRAY_BUFFER, null);
			});
		};
		if (args) obj.initCube.apply(obj, args);
		return obj;
	};
	(module.exports = Cube).create(cube);
}).call(this);
});
api.provide("/shapes/Cube2", function (require, module, exports) {
/*global module require Uint8Array Float32Array*/
(function(){"use strict";
	var util = require('../base/util'),
	    Entity = require('../scene/Entity'),
	    BasicMaterial = require('../materials/BasicMaterial');
	function Cube2 (world) {return this.initCube2(world);}
	util.inherits(Cube2, Entity);
	var cube2 = Cube2.prototype;
	Cube2.check = function (obj) {
		return Entity.check(obj) && obj.initCube2 === cube2.initCube2;
	};
	Cube2.create = function (obj, args) {
		var supr = Entity.prototype;
		obj = obj || {};
		if (!Entity.check(obj)) {
			Entity.create(obj);
		}
		obj.initCube2 = cube2.initCube2 || function (parent, material) {
			material = material || new BasicMaterial(parent.world);
			return this.initEntity(parent, material).allocate();
		};
		obj.allocate = function () {
			var uvData = [//uv square
				[ 0.0, 1.0], [ 1.0, 1.0],
				[ 1.0, 0.0], [ 0.0, 0.0]
			];
			return this.polygonMeshFromJSON({
				vertices: [
					//back
					[-1.0,-1.0,-1.0],//v0
					[ 1.0,-1.0,-1.0],//v1
					[ 1.0, 1.0,-1.0],//v2
					[-1.0, 1.0,-1.0],//v3
					//front
					[-1.0,-1.0, 1.0],//v4
					[ 1.0,-1.0, 1.0],//v5
					[ 1.0, 1.0, 1.0],//v6
					[-1.0, 1.0, 1.0] //v7
				],
				surfaces: [
					{polygons: [//cap
						{vertices: [0, 1, 2, 3], groups: {uv: uvData, color: [1.0, 0.0, 0.0, 1.0]}}
					]},
					{polygons: [//body
						{vertices: [0, 1, 4, 5], groups: {uv: uvData, color: [0.0, 1.0, 0.0, 1.0]}},
						{vertices: [1, 2, 6, 5], groups: {uv: uvData, color: [0.0, 1.0, 1.0, 1.0]}},
						{vertices: [2, 3, 7, 6], groups: {uv: uvData, color: [1.0, 1.0, 0.0, 1.0]}},
						{vertices: [3, 0, 4, 7], groups: {uv: uvData, color: [1.0, 0.0, 1.0, 1.0]}}
					]},
					{polygons: [//cap
						{vertices: [4, 5, 6, 7], groups: {uv: uvData, color: [0.0, 0.0, 1.0, 1.0]}}
					]}
				],
				groups: {materials: []}
			});
		};
		obj.dellocate = function () {
			this.emptyPolygonMesh();
			return this;
		};
		if (args) obj.initCube2.apply(obj, args);
		return obj;
	};
	(module.exports = Cube2).create(Cube2.prototype);
}).call(this);
});
(function(){"use strict";
	/*global module name api*/
	if (typeof module !== 'undefined') {
		module.exports = api.freeze();
	} else {
		api.freeze("graffiti", this);
	}
}).call(this);
}).call(this);