/*global module require client*/
(function(){"use strict";
	var util = require('util'),
	    Emitter = client.require('core').require('Emitter');
	function Obj3d (parent) {return this.initObj3d(parent);}
	util.inherits(Obj3d, Emitter);
	var obj3d = Obj3d.prototype;
	Obj3d.check = function (obj) {
		return Emitter.check(obj) && obj.initObj3d === obj3d.initObj3d;
	};
	Obj3d.create = function (obj) {
		obj = obj || {};
		if (!Emitter.check(obj)) {
			Emitter.create(obj);
		}
		obj.initObj3d = obj3d.initObj3d || function (parent) {
			this.world = this;
			this.children = [];
			if (parent) parent.add(this);
			return this;
		};
		obj.uid = function (refresh) {
			var charCode = util.randomCharCode;
			if (refresh) delete this._uid;
			return (this._uid = this._uid || String.fromCharCode(
				charCode(), charCode(), charCode(), charCode()
			) + (+new Date()));
		};
		obj.add = function (obj) {
			if (!Obj3d.check(obj)) return this;
			if (obj !== this) {
				if (obj.parent) {
					if (obj.parent === this) {
						return this;
					}
					obj.parent.remove(obj);
				}
				obj.index = this.children.length;
				this.children.push(obj);
				obj.parent = this;
			}
			obj.world = this.world || this;
			return this;
		};
		obj.remove = function (obj) {
			var index;
			if (!obj || obj.parent !== this) return this;
			if (obj.index && this.children[obj.index] === obj) {
				index = obj.index;
			} else {
				index = this.children.indexOf(obj);
			}
			this.children.splice(index, 1);
			delete obj.index;
			delete obj.parent;
			delete obj.world;
			this.children.forEach(function (obj, ind) {
				if (ind >= index) {
					obj.index = ind;
				}
			});
			return this;
		};
		obj.enumerate = function (fn) {
			this.children.forEach(fn, this);
			return this;
		};
		obj.enumAsync = function (fn, cb) {
			var that = this,
			    count = this.children.length;
			function end () {
				count -= 1;
				if (count === 0) {
					cb.call(that, arguments);
				}
			}
			return this.children.forEach(function (val, ind, obj) {
				fn.call(this, end, val, ind, obj);
			}, this);
		};
		obj.enumInvoke = function (name, args) {
			this.children.forEach(function (obj) {
				obj[name].apply(obj, args);
			});
			return this;
		};
		obj.enumInvokeAsync = function (name, cb, args) {
			args = args || [];
			args.unshift(null);
			return this.enumAsync(function (end, obj) {
				args[0] = end;
				obj[name].apply(obj, args);
			}, cb);
		};
		obj.load = function (cb) {
			util.safeApply(cb, this);
			return this;
		};
		obj.buffer = function () {
			return this;
		};
		obj.render = function () {
			return this;
		};
		obj.update = function () {
			return this;
		};
		return obj;
	};
	(module.exports = Obj3d).create(obj3d);
}).call(this);