/*global module require client window Image*/
(function(){"use strict";
	var util = require('util'),
	    Emitter = client.require('core').require('Emitter');
	function WebGL (canvas, opts) {return this.initWebGL(canvas, opts);}
	var webGL = WebGL.prototype;
	util.inherits(WebGL, Emitter);
	WebGL.check = function (obj) {
		return Emitter.check(obj) && obj.initWebGL === webGL.initWebGL;
	};
	var getWebGLContext = (function(){
		var names = ["webgl", "experimental-webgl", "webkit-3d", "moz-webgl"],
		    tick = 1000 / 60,
		    len = names.length,
		    win = window;
		win.requestAnimFrame =
			win.requestAnimationFrame ||
			win.webkitRequestAnimationFrame ||
			win.mozRequestAnimationFrame ||
			win.oRequestAnimationFrame ||
			win.msRequestAnimationFrame ||
			function (cb) {
				win.setTimeout(cb, tick);
			};
		return function (canvas, opts) {
			var context, i = 0;
			for (; i < len; i += 1) {
				try {
					context = canvas.getContext(names[i], opts);
				} catch(e) {}
				if (context) break;
			}
			return context;
		};
	}).call(this);
	WebGL.create = function (obj, args) {
		obj = obj || {};
		if (!Emitter.check(obj)) {
			Emitter.create(obj);
		}
		obj.initWebGL = webGL.initWebGL || function (canvas, opts) {
			this.gl = getWebGLContext(canvas, opts);
			return this.initEmitter();
		};
		obj.createVertexShader = function (src) {
			var gl = this.gl,
			    shdr = gl.createShader(gl.VERTEX_SHADER);
			return this.compileShader(shdr, src, 'vertex');
		};
		obj.createFragmentShader = function (src) {
			var gl = this.gl,
			    shdr = gl.createShader(gl.FRAGMENT_SHADER);
			return this.compileShader(shdr, src, 'fragment');
		};
		obj.compileShader = function (shdr, src, type) {
			if (typeof src !== 'string' && src && src.length && src.join) {
				src = src.join('\n');
			}
			var gl = this.gl,
			    err;
			gl.shaderSource(shdr, src);
			gl.compileShader(shdr);
			if (!gl.getShaderParameter(shdr, gl.COMPILE_STATUS)) {
				err = gl.getShaderInfoLog(shdr);
				gl.deleteShader(shdr);
				throw err;
			}
			this.emit('newShader', shdr, type);
			return shdr;
		};
		obj.createProgram = function (vShdr, fShdr) {
			var gl = this.gl,
			    prgm = gl.createProgram(),
			    err;
			gl.attachShader(prgm, vShdr);
			gl.attachShader(prgm, fShdr);
			gl.linkProgram(prgm);
			if (!gl.getProgramParameter(prgm, gl.LINK_STATUS)) {
				err = gl.getProgramInfoLog(prgm);
				gl.deleteProgram(prgm);
				throw err;
			}
			this.emit('newProgram', prgm);
			return prgm;
		};
		obj.createArrayBuffer = function (data, cleanup, usage) {
			var gl = this.gl;
			return this.createBufferData(
				gl.ARRAY_BUFFER, data, usage || gl.STATIC_DRAW, cleanup);
		};
		obj.createElementArrayBuffer = function (data, cleanup, usage) {
			var gl = this.gl;
			return this.createBufferData(
				gl.ELEMENT_ARRAY_BUFFER, data, usage || gl.STATIC_DRAW, cleanup);
		};
		obj.createBufferData = function (targ, data, usage, cleanup) {
			var gl = this.gl,
			    buff = gl.createBuffer();
			gl.bindBuffer(targ, buff);
			gl.bufferData(targ, data, usage);
			if (cleanup) gl.bindBuffer(targ, null);
			this.emit('newBuffer', buff, targ, data, usage);
			return buff;
		};
		obj.extractProgramAttrs = function (prgm, obj, attrs) {
			obj = obj || {};
			var gl = this.gl,
			    len = attrs.length, i = 0,
			    attr;
			for (; i < len; i += 1) {
				attr = obj[attrs[i]] = gl.getAttribLocation(prgm, attrs[i]);
			}
			return obj;
		};
		obj.extractProgramUniforms = function (prgm, obj, uniforms) {
			obj = obj || {};
			var gl = this.gl,
			    len = uniforms.length, i = 0;
			for (; i < len; i += 1) {
				obj[uniforms[i]] = gl.getUniformLocation(prgm, uniforms[i]);
			}
			return obj;
		};
		obj.loadTexture = function (url, cb) {
			var gl = this.gl,
			    texture = gl.createTexture(),
			    image = texture.image = new Image();
			image.onload = function () {
				gl.bindTexture(gl.TEXTURE_2D, texture);
				gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, image);
				gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
				gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR);//_MIPMAP_LINEAR);
				gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
				gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
				gl.bindTexture(gl.TEXTURE_2D, null);
				util.safeApply(cb, this, [texture, image]);
			};
			image.src = url;
			return texture;
		};
		if (args) obj.initWebGL.apply(obj, args);
		return obj;
	};
	(module.exports = WebGL).create(webGL);
}).call(this);