/*global module require*/
(function(){"use strict";
	var util = require('../base/util');
	function PolygonMesh (json) {return this.initPolygonMesh(json);}
	function NodeList (owner) {return NodeList.create(null, [owner]);}
	function Triangle () {return this.initPolygonMeshPolygon();}
	function Surface () {return this.initPolygonMeshSurface();}
	function Polygon () {return this.initPolygonMeshPolygon();}
	function Vertex () {return this.initPolygonMeshVertex();}
	function Group () {return this.initPolygonMeshGroup();}
	function Edge () {return this.initPolygonMeshEdge();}
	function Node () {return this.initPolygonMeshNode();}
	util.inherits(Triangle, Node);
	util.inherits(Surface, Node);
	util.inherits(Polygon, Node);
	util.inherits(Vertex, Node);
	util.inherits(Group, Node);
	util.inherits(Edge, Node);
	var Vector3 = require('/glMath').require('Vector3'),
	    polygonMesh = PolygonMesh.prototype,
	    nodeList = NodeList.prototype,
	    triangle = Triangle.prototype,
	    surface = Surface.prototype,
	    polygon = Polygon.prototype,
	    vertex = Vertex.prototype,
	    group = Group.prototype,
	    edge = Edge.prototype,
	    node = node.prototype;
	PolygonMesh.check = function (obj) {
		return obj && obj.initPolygonMesh === polygonMesh.initPolygonMesh;
	};
	NodeList.check = function (obj) {
		return obj && obj.initPolygonMeshNodeList === nodeList.initPolygonMeshNodeList;
	};
	Triangle.check = function (obj) {
		return Node.check(obj) && obj.initPolygonMeshTriangle === triangle.initPolygonMeshTriangle;
	};
	Surface.check = function (obj) {
		return Node.check(obj) && obj.initPolygonMeshSurface === surface.initPolygonMeshSurface;
	};
	Polygon.check = function (obj) {
		return Node.check(obj) && obj.initPolygonMeshPolygon === polygon.initPolygonMeshPolygon;
	};
	Vertex.check = function (obj) {
		return Node.check(obj) && obj.initPolygonMeshVertex === vertex.initPolygonMeshVertex;
	};
	Group.check = function (obj) {
		return Node.check(obj) && obj.initPolygonMeshGroup === group.initPolygonMeshGroup;
	};
	Edge.check = function (obj) {
		return Node.check(obj) && obj.initPolygonMeshEdge === edge.initPolygonMeshEdge;
	};
	Node.check = function (obj) {
		return obj && obj.initPolygonMeshNode === node.initPolygonMeshNode;
	};
	PolygonMesh.create = function (obj, args) {
		obj = obj || {};
		obj.initPolygonMesh = polygonMesh.initPolygonMesh || function (json) {
			this.meshVertices = new NodeList(this, Vertex.check);
			this.meshSurfaces = new NodeList(this, Surface.check);
			return this.polygonMeshFromJSON(json);
		};
		obj.polygonMeshFromJSON = function (json) {
			if (!json) return this;
			if (typeof json === 'string') {
				json = JSON.parse(json);
			}
			this.meshVertices.set(json.vertices, function (vertex) {
				return new Vertex(this).fromJSON(vertex);
			});
			this.meshSurfaces.set(json.polygons, function (surface) {
				return new Surface(this).fromJSON(surface);
			});
			return this.computePolygonMesh();
		};
		obj.polygonMeshToJSON = function () {
			return JSON.stringify({
				vertices: this.meshVertices.toJSON(),
				surfaces: this.meshSurfaces.toJSON()
			});
		};
		obj.emptyPolygonMesh = function () {
			this.meshVertices.set([]);
			this.meshSurfaces.set([]);
			return this;
		};
		obj.computePolygonMesh = function () {
			return this;
		};
		obj.bufferPolygonMesh = function () {
			return this;
		};
		obj.renderPolygonMesh = function () {
			return this;
		};
		if (args) obj.initPolygonMesh.apply(obj, args);
		return obj;
	};
	NodeList.create = function (obj, args) {
		obj = obj || [];
		obj.initPolygonMeshNodeList = nodeList.initPolygonMeshNodeList || function (owner, check) {
			this.owner = this;
			this.check = check || Node.check;
			return this;
		};
		obj.enumerate = function (fn) {
			return this.forEach(fn, this);
		};
		obj.find = function (child) {
			if (!this.check(child)) return -1;
			return this.indexOf(child);
		};
		obj.add = function (child) {
			if (!~this.find(child)) this.push(child);
			return this;
		};
		obj.remove = function (child)  {
			var index = this.find(child);
			if (~index) this.splice(index, 1);
			return this;
		};
		obj.set = function (list, fn) {
			while (this[0]) this.pop();
			if (list && list.forEach) {
				if (typeof fn !== 'function') {
					fn = function (val) {
						return val;
					};
				}
				list.forEach(function (val, ind, obj) {
					this[ind] = fn.call(this, val, ind, obj);
				}, this);
			}
			return this;
		};
		obj.toJSON = function () {
			var list = [];
			this.enumerate(function (node) {
				var json = node && node.toJSON && node.toJSON();
				if (json) list.push(json);
			});
			return '[' + list.join(',') + ']';
		};
		if (args) obj.initNodeList.apply(obj, args);
		return obj;
	};
	Triangle.create = function (obj, args) {
		obj = obj || {};
		if (!Node.check(obj)) {
			Node.create(obj);
		}
		obj.initPolygonMeshTriangle = triangle.initPolygonMeshTriangle || function (polygon) {
			this.adjacent = new NodeList(this, Triangle.check);
			this.vertices = new NodeList(this, Vertex.check);
			return this.initPolygonMeshNode(polygon);
		};
		obj.compute = function () {
			return this;
		};
		obj.fromJSON = function (json) {
			if (!json) return this;
			if (typeof json === 'string') {
				json = JSON.parse(json);
			}
			var that = this;
			this.vertices.set(json.vertices, function (vertex) {
				return new Vertex().fromJSON(vertex).addTo(that);
			});
			return this;
		};
		obj.toJSON = function () {
			return JSON.stringify({
				vertices: this.vertices.toJSON()
			});
		};
		if (args) obj.initTriangle.apply(obj, args);
		return obj;
	};
	Surface.create = function (obj, args) {
		obj = obj || {};
		if (!Node.check(obj)) {
			Node.create(obj);
		}
		obj.initPolygonMeshSurface = surface.initPolygonMeshSurface || function (mesh) {
			this.adjacent = new NodeList(this, Surface.check);
			this.polygons = new NodeList(this, Polygon.check);
			this.groups = new NodeList(this, Group.check);
			return this.initPolygonMeshNode(mesh);
		};
		obj.compute = function () {
			return this;
		};
		obj.fromJSON = function (json) {
			if (!json) return this;
			if (typeof json === 'string') {
				json = JSON.parse(json);
			}
			var that = this;
			this.polygons.set(json.polygons, function (polygon) {
				return new Polygon(this).fromJSON(polygon).addTo(that);
			});
			this.groups.set(json.groups, function (group) {
				return new Group(this).fromJSON(group).addTo(that);
			});
			return this;
		};
		obj.toJSON = function () {
			return JSON.stringify({
				polygons: this.polygons.toJSON(),
				groups: this.groups.toJSON()
			});
		};
		if (args) obj.initSurface.apply(obj, args);
		return obj;
	};
	Polygon.create = function (obj, args) {
		obj = obj || {};
		if (!Node.check(obj)) {
			Node.create(obj);
		}
		obj.initPolygonMeshPolygon = polygon.initPolygonMeshPolygon || function (surface) {
			this.triangles = new NodeList(this, Triangle.check);
			this.adjacent = new NodeList(this, Polygon.check);
			this.edges = new NodeList(this, Edge.check);
			return this.initPolygonMeshNode(surface);
		};
		obj.compute = function () {
			return this;
		};
		obj.fromJSON = function (json) {
			if (!json) return this;
			if (typeof json === 'string') {
				json = JSON.parse(json);
			}
			var that = this;
			this.edges.set(json.edges, function (edge) {
				return new Edge(this).fromJSON(edge).addTo(that);
			});
			return this;
		};
		obj.toJSON = function () {
			return JSON.stringify({
				edges: this.edges.toJSON()
			});
		};
		if (args) obj.initPolygon.apply(obj, args);
		return obj;
	};
	Vertex.create = function (obj, args) {
		obj = obj || {};
		if (!Node.check(obj)) {
			Node.create(obj);
		}
		obj.initPolygonMeshVertex = vertex.initPolygonMeshVertex || function (edge) {
			this.triangles = new NodeList(this, Triangle.check);
			this.adjacent = new NodeList(this, Vertex.check);
			this.surfaces = new NodeList(this, Surface.check);
			this.polygons = new NodeList(this, Polygon.check);
			this.edges = new NodeList(this, Edge.check);
			this.vec = new Vector3();
			return this.initPolygonMeshNode(edge);
		};
		obj.compute = function () {
			return this;
		};
		obj.fromJSON = function (json) {
			if (!json) return this;
			if (typeof json === 'string') {
				json = JSON.parse(json);
			}
			return this.vec.set(json);
		};
		obj.toJSON = function () {
			return '[' + this.vec.str() + ']';
		};
		if (args) obj.initVertex.apply(obj, args);
		return obj;
	};
	Group.create = function (obj, args) {
		obj = obj || {};
		obj.initPolygonMeshGroup = group.initPolygonMeshGroup || function (surface) {
			return this.initPolygonMeshNode(surface);
		};
		obj.compute = function () {
			return this;
		};
		obj.fromJSON = function (json) {
			return this;
		};
		obj.toJSON = function () {
			var json = '';
			return json;
		};
		if (args) obj.initGroup.apply(obj, args);
		return obj;
	};
	Edge.create = function (obj, args) {
		obj = obj || {};
		if (!Node.check(obj)) {
			Node.create(obj);
		}
		obj.initPolygonMeshEdge = edge.initPolygonMeshEdge || function (triangle) {
			this.adjacent = new NodeList(this, Edge.check);
			this.polygons = new NodeList(this, Polygon.check);
			this.vertices = new NodeList(this, Vertex.check);
			this.edges = new NodeList(this, Edge.check);
			return this.initPolygonMeshNode(triangle);
		};
		obj.compute = function () {
			return this;
		};
		obj.fromJSON = function (json) {
			if (!json) return this;
			if (typeof json === 'string') {
				json = JSON.parse(json);
			}
			var that = this;
			this.vertices.set(json.vertices, function (vertex) {
				return new Vertex().fromJSON(vertex).addTo(that);
			});
			return this;
		};
		obj.toJSON = function () {
			return JSON.stringify({
				vertices: this.vertices.toJSON()
			});
		};
		if (args) obj.initEdge.apply(obj, args);
		return obj;
	};
	Node.create = function (obj, args) {
		obj = obj || {};
		obj.initPolygonMeshNode = node.initPolygonMeshNode || function () {
			this.parents = new NodeList(this);
			this.children = new NodeList(this);
			return this;
		};
		obj.addTo = function (parent) {
			if (Node.check(parent)) {
				parent.children.add(this);
				this.parents.add(parent);
			}
			return this;
		};
		obj.add = function (child) {
			if (Node.check(child)) {
				this.children.add(child);
				child.parents.add(this);
			}
			return this;
		};
		obj.remove = function (child) {
			if (Node.check(child)) {
				this.children.remove(child);
				child.parents.remove(this);
			}
			return this;
		};
		if (args) obj.initNode.apply(obj, args);
		return obj;
	};
	(module.exports = PolygonMesh).create(polygonMesh);
	(module.exports = NodeList).create(nodeList);
	(module.exports = Triangle).create(triangle);
	(module.exports = Surface).create(surface);
	(module.exports = Polygon).create(polygon);
	(module.exports = Vertex).create(vertex);
	(module.exports = Group).create(group);
	(module.exports = Edge).create(edge);
	(module.exports = Node).create(node);
}).call(this);