/*global module require Float32Array Float64Array*/
(function(){"use strict";
	var util = require('../base/util'),
	    glMath = require('../glMath'),
	    Vector3 = glMath.require('Vector3'),
	    Matrix3 = glMath.require('Matrix3'),
	    Matrix4 = glMath.require('Matrix4'),
	    Obj3d = require('../base/Obj3d'),
	    Movable = require('../behavior/Movable'),
	    Animated = require('../behavior/Animated');
	glMath = glMath.require('utils');
	function Camera (world) {return this.initCamera(world);}
	util.inherits(Camera, Obj3d);
	var camera = Camera.prototype;
	Camera.check = function (obj) {
		return Obj3d.check(obj) && obj.initCamera === camera.initCamera;
	};
	Camera.create = function (obj, args) {
		obj = obj || {};
		if (!Obj3d.check(obj)) {
			Obj3d.create(obj);
		}
		obj.initCamera = camera.initCamera || function (world) {
			if (!world) return this;
			this.initObj3d(world);
			this.mvpNormal = Matrix4.create(new Float32Array(16));
			this.mvp = Matrix4.create(new Float32Array(16));
			this.view = new Matrix4();
			this.projection = new Matrix4();
			this.target = new Vector3([0.0, 0.0, 0.0]);
			this.up = new Vector3([0.0, 1.0, 0.0]);
			this.eye = new Vector3([0.0, 2.0, 10.0]);
			this.far = 1000.0;
			this.near = 0.001;
			this.fovy = 45.0;
			this.aspect = 0.75;
			return this;
		};
		obj.projectLookAt = function () {
			this.view.lookAt(this.eye, this.target, this.up);
			this.projection.perspective(this.fovy, this.aspect, this.near, this.far);
			return this;
		};
		obj.flatten = function (model) {
			var world = this.world,
			    attrs = world.shader.attrs,
			    mvpNormal = this.mvpNormal;
			this.mvp.set(this.projection).multiply(this.view).multiply(model);
			mvpNormal.set(this.view).multiply(model).inverse().transpose();
			world.gl.uniformMatrix4fv(attrs.mvp, false, this.mvp);
			world.gl.uniformMatrix4fv(attrs.mvpNormal, false, mvpNormal);
			return this;
		};
		obj.moveForward = function (scale) {
			var dir = this.target.copy().
				direction(this.eye).
				scale(scale);
			this.target.add(dir);
			this.eye.add(dir);
			return this;
		};
		obj.moveUp = function (scale) {
			var dir, right = this.target.copy().
				direction(this.eye).
				cross(this.up);
			dir = this.target.copy().
				direction(this.eye).
				cross(right).
				negate().
				scale(scale);
			this.target.add(dir);
			this.eye.add(dir);
			return this;
		};
		obj.moveRight = function (scale) {
			var dir = this.target.copy().
				direction(this.eye).
				cross(this.up).
				scale(scale);
			this.target.add(dir);
			this.eye.add(dir);
			return this;
		};
		obj.orbitPitch = function (deg) {
			var dir = this.target.copy().
				direction(this.eye).
				cross(this.up);
			new Matrix4().identity().
				rotate(glMath.degToRad(deg), dir).
				multiplyVec3(this.eye).
				multiplyVec3(this.up);
			return this;
		};
		obj.orbitRoll = function (deg) {
			var dir = this.target.copy().
				direction(this.eye);
			new Matrix4().identity().
				rotate(glMath.degToRad(deg), dir).
				multiplyVec3(this.eye).
				multiplyVec3(this.up);
			return this;
		};
		obj.orbitYaw = function (deg) {
			var dir, right = this.target.copy().
				direction(this.eye).
				cross(this.up);
			dir = this.target.copy().
				direction(this.eye).
				cross(right);
			new Matrix4().identity().
				rotate(glMath.degToRad(deg), dir).
				multiplyVec3(this.eye).
				multiplyVec3(this.up);
			return this;
		};
		obj.lookOrbitPitch = function (deg) {
			return this;
		};
		obj.lookOrbitYaw = function (deg) {
			return this;
		};
		if (args) obj.initCamera.apply(obj, args);
		return obj;
	};
	(module.exports = Camera).create(camera);
}).call(this);