/*global module require*/
(function(){"use strict";
	var util = require('../base/util'),
	    Vector3 = require('../glMath').require('Vector3'),
	    Obj3d = require('../base/Obj3d'),
	    Shaded = require('../behavior/Shaded'),
	    Movable = require('../behavior/Movable'),
	    Colored = require('../behavior/Colored'),
	    Animated = require('../behavior/Animated'),
	    PolygonMesh = require('../behavior/PolygonMesh'),
	    ParticleField = require('../behavior/ParticleField');
	function Entity (world, material) {return this.initEntity(world, material);}
	var entity = Entity.prototype;
	util.inherits(Entity, Obj3d);
	Entity.check = function (obj) {
		return Obj3d.check(obj) && obj.initEntity === entity.initEntity;
	};
	Entity.create = function (obj, args) {
		obj = obj || {};
		function inherit (constr) {
			if (!constr.check(obj)) constr.create(obj);
		}
		inherit(Obj3d);
		inherit(Shaded);
		inherit(Movable);
		inherit(Colored);
		inherit(Animated);
		inherit(PolygonMesh);
		inherit(ParticleField);
		obj.initEntity = entity.initEntitiy || function (parent, material) {
			this.initObj3d(parent);
			this.initShaded(material);
			this.initMovable();
			this.initColored();
			this.initAnimated();
			this.initPolygonMesh();
			this.initParticleField();
			return this;
		};
		obj.update = function () {
			this.updateMoveable();
			return this;
		};
		obj.buffer = function () {
			this.bufferPolygonMesh();
			return this;
		};
		obj.render = function (cb) {
			this.renderMovable();
			this.renderShaded();
			this.renderPolygonMesh();
			util.safeApply(cb, this);
			return this;
		};
		if (args) obj.initEntity.apply(obj, args);
		return obj;
	};
	(module.exports = Entity).create(entity);
}).call(this);