/*global module require*/
(function(){"use strict";
	var util = require('../base/util');
	function Shader (world) {return this.initShader(world);}
	var shader = Shader.prototype;
	Shader.check = function (obj) {
		return obj && obj.initShader === shader.initShader;
	};
	Shader.create = function (obj, args) {
		obj = obj || {};
		obj.initShader = shader.initShader || function (world, src) {
			this.world = world;
			this.attrs = {};
			return this.build(src).setup();
		};
		obj.build = function (src) {
			if (this.built) return this;
			var webgl = this.world.webgl;
			this.fragment = webgl.createFragmentShader(src.fragment);
			this.vertex = webgl.createVertexShader(src.vertex);
			this.program = webgl.createProgram(this.fragment, this.vertex);
			this.built = true;
			delete this.ready;
			return this;
		};
		obj.rebuild = function (frag, vert) {
			delete this.built;
			return this.build(frag, vert);
		};
		obj.setup = function (cb) {
			if (this.built && !this.ready) {
				util.safeApply(cb, this,
					[this.world.webgl, this.world.gl, this.attrs, this.program]);
				this.ready = true;
			}
			return this;
		};
		obj.resetup = function () {
			delete this.ready;
			return this.setup();
		};
		obj.use = function (modelMatrix) {
			var world = this.world;
			if (this.built && this.ready) {
				if (world.shader === this) {
					world.camera.flatten(modelMatrix);
					return this;
				}
				world.gl.useProgram(this.program);
				world.shader = this;
				world.camera.flatten(modelMatrix);
			}
			return this;
		};
		if (args) obj.initShader.apply(obj, args);
		return obj;
	};
	(module.exports = Shader).create(shader);
}).call(this);