/*global module require*/
(function(){"use strict";
	var util = require('../base/util'),
	    glMath = require('../glMath'),
	    Vector3 = glMath.require('Vector3'),
	    Vector4 = glMath.require('Vector4'),
	    Obj3d = require('../base/Obj3d'),
	    Camera = require('../scene/Camera');
	function World (app) {return this.initWorld(app);}
	util.inherits(World, Obj3d);
	var world = World.prototype;
	World.check = function (obj) {
		return Obj3d.check(obj) && obj.initWorld === world.initWorld;
	};
	World.create = function (obj, args) {
		obj = obj || {};
		if (!Obj3d.check(obj)) {
			Obj3d.create(obj);
		}
		obj.initWorld = world.initWorld || function (app) {
			this.gl = app.webgl.gl;
			this.app = app;
			this.webgl = app.webgl;
			this.initObj3d(this);
			this.camera = new Camera(this);
			this.lightDir = new Vector3([0.0, 0.5, 0.5]);
			this.sampler2d = 0;
			this.clearColor = new Vector4([0.0, 0.0, 0.0, 1.0]);
			this.clearDepth = 1.0;
			return this;
		};
		obj.load = function (cb) {
			return this.enumInvokeAsync('load', cb, [this.webgl, this.gl]);
		};
		obj.buffer = function () {
			var gl = this.gl,
			    c = this.clearColor;
			gl.clearColor(c[0], c[1], c[2], c[3]);
			gl.clearDepth(1.0);
			gl.enable(gl.BLEND);
			gl.enable(gl.CULL_FACE);
			gl.enable(gl.DEPTH_TEST);
			gl.enable(gl.TEXTURE_2D);
			this.enumInvoke('buffer', [this.webgl, this.gl]);
			return this;
		};
		obj.reshape = function () {
			var app = this.app;
			this.camera.aspect = app.width / app.height;
			this.gl.viewport(0, 0, app.width, app.height);
			return this;
		};
		obj.render = function () {
			this.clear();
			this.camera.projectLookAt();
			this.enumInvoke('render', [this.webgl, this.gl]);
			this.gl.flush();
			this.gl.finish();
			return this;
		};
		obj.clear = function () {
			this.gl.clear(this.gl.COLOR_BUFFER_BIT | this.gl.DEPTH_BUFFER_BIT);
			return this;
		};
		obj.pick = function (x, y) {
			return this;
		};
		if (args) obj.initWorld.apply(obj, args);
		return obj;
	};
	(module.exports = World).create(world);
}).call(this);