/*global module require*/
(function(){"use strict";
	var util = require('../base/util'),
	    Shader = require('../scene/Shader');
	function BasicShader (world) {return this.initBasicShader(world);}
	util.inherits(BasicShader, Shader);
	var basicShader = BasicShader.prototype;
	BasicShader.check = function (obj) {
		return Shader.check(obj) && obj.initBasicShader === basicShader.initBasicShader;
	};
	BasicShader.create = function (obj, args) {
		var supr = Shader.prototype;
		obj = obj || {};
		if (!Shader.check(obj)) {
			Shader.create(obj);
		}
		obj.initBasicShader = basicShader.initBasicShader || function (world) {
			return this.initShader(world, {
				fragment: [
					'#ifdef GL_ES',
					'precision mediump float;',
					'#endif',
					'uniform sampler2D sampler2d;',
					'varying float myDot;',
					'varying vec4 myColor;',
					'varying vec2 myTexCoord;',
					'void main(void) {',
						'vec2 texCoord = vec2(myTexCoord.s, 1.0 - myTexCoord.t);',
						'vec4 color = texture2D(sampler2d, texCoord) / 2.0;',
						'color += myColor / 2.0;',
						'color /= 0.5;',
						'gl_FragColor = vec4(color.xyz * myDot, color.a);',
					'}'
				],
				vertex: [
					'uniform mat4 mvp;',
					'uniform mat4 mvpNormal;',
					'uniform vec3 lightDir;',
					'attribute vec3 position;',
					'attribute vec2 texCoord;',
					'attribute vec3 normal;',
					'attribute vec4 color;',
					'varying float myDot;',
					'varying vec4 myColor;',
					'varying vec2 myTexCoord;',
					'void main(void) {',
						'gl_Position = mvp * vec4(position, 1.0);',
						'myColor = color;',
						'myTexCoord = texCoord;',
						'vec4 transNormal = mvpNormal * vec4(normal, 1.0);',
						'myDot = max(dot(transNormal.xyz, lightDir), 0.1);',
					'}'
				]
			});
		};
		obj.setup = function () {
			return supr.setup.call(this, function (webgl, gl, attrs, program) {
				webgl.extractProgramUniforms(program, attrs, ['mvp', 'mvpNormal', 'lightDir', 'sampler2d']);
				webgl.extractProgramAttrs(program, attrs, ['position', 'texCoord', 'normal', 'color']);
			});
		};
		obj.use = function (modelMatrix) {
			var gl = this.world.gl,
			    ld = this.world.lightDir,
			    attrs = this.attrs;
			supr.use.call(this, modelMatrix);
			gl.uniform3f(attrs.lightDir, ld[0], ld[1], ld[2]);
			gl.uniform1i(attrs.sampler2d, this.world.sampler2d);
			return this;
		};
		if (args) obj.initBasicShader.apply(obj, args);
		return obj;
	};
	(module.exports = BasicShader).create(basicShader);
}).call(this);