/*global module require Uint8Array Float32Array*/
(function(){"use strict";
	var util = require('../base/util'),
	    Entity = require('../scene/Entity'),
	    BasicMaterial = require('../materials/BasicMaterial');
	function Cube (world) {return this.initCube(world);}
	util.inherits(Cube, Entity);
	var cube = Cube.prototype;
	Cube.check = function (obj) {
		return Entity.check(obj) && obj.initCube === cube.initCube;
	};
	Cube.create = function (obj, args) {
		var supr = Entity.prototype;
		obj = obj || {};
		if (!Entity.check(obj)) {
			Entity.create(obj);
		}
		obj.initCube = cube.initCube || function (parent, material) {
			material = material || new BasicMaterial(parent.world);
			return this.initEntity(parent, material).allocate();
		};
		obj.allocate = function () {
			return this.
				giveIndices().
				givePoints().
				giveNormals().
				giveColors();
		};
		obj.dellocate = function () {
			delete this.indices;
			delete this.points;
			delete this.normals;
			delete this.colors;
			return this;
		};
		obj.giveIndices = function () {
			// v0-v1-v2-v3 front
			// v0-v3-v4-v5 right
			// v0-v5-v6-v1 top
			// v1-v6-v7-v2 left
			// v7-v4-v3-v2 bottom
			// v4-v7-v6-v5 back
			if (this.indices) return this;
			this.indices = new Uint8Array([
				 0,  1,  2,   0,  2,  3,
				 4,  5,  6,   4,  6,  7,
				 8,  9, 10,   8, 10, 11,
				12, 13, 14,  12, 14, 15,
				16, 17, 18,  16, 18, 19,
				20, 21, 22,  20, 22, 23
			]);
			this.indices.units = this.indices.length;
			this.indices.size = 1;
			return this;
		};
		obj.givePoints = function () {
			if (this.points) return this;
			this.points = new Float32Array([
				 1.0, 1.0, 1.0,  1.0, 1.0,  -1.0, 1.0, 1.0,  0.0, 1.0,  -1.0,-1.0, 1.0,  0.0, 0.0,   1.0,-1.0, 1.0,  1.0, 0.0,
				 1.0, 1.0, 1.0,  0.0, 1.0,   1.0,-1.0, 1.0,  0.0, 0.0,   1.0,-1.0,-1.0,  1.0, 0.0,   1.0, 1.0,-1.0,  1.0, 1.0,
				 1.0, 1.0, 1.0,  1.0, 0.0,   1.0, 1.0,-1.0,  1.0, 1.0,  -1.0, 1.0,-1.0,  0.0, 1.0,  -1.0, 1.0, 1.0,  0.0, 0.0,
				-1.0, 1.0, 1.0,  1.0, 1.0,  -1.0, 1.0,-1.0,  0.0, 1.0,  -1.0,-1.0,-1.0,  0.0, 0.0,  -1.0,-1.0, 1.0,  1.0, 0.0,
				-1.0,-1.0,-1.0,  0.0, 0.0,   1.0,-1.0,-1.0,  1.0, 0.0,   1.0,-1.0, 1.0,  1.0, 1.0,  -1.0,-1.0, 1.0,  0.0, 1.0,
				 1.0,-1.0,-1.0,  0.0, 0.0,  -1.0,-1.0,-1.0,  1.0, 0.0,  -1.0, 1.0,-1.0,  1.0, 1.0,   1.0, 1.0,-1.0,  0.0, 1.0
			]);
			this.points.units = this.points.length / (this.points.size = 5);
			this.points.vertexSize = 3;
			this.points.texCoordSize = 2;
			return this;
		};
		obj.giveNormals = function () {
			if (this.normals) return this;
			this.normals = new Float32Array([
				 0.0, 0.0, 1.0,   0.0, 0.0, 1.0,   0.0, 0.0, 1.0,   0.0, 0.0, 1.0,
				 1.0, 0.0, 0.0,   1.0, 0.0, 0.0,   1.0, 0.0, 0.0,   1.0, 0.0, 0.0,
				 0.0, 1.0, 0.0,   0.0, 1.0, 0.0,   0.0, 1.0, 0.0,   0.0, 1.0, 0.0,
				-1.0, 0.0, 0.0,  -1.0, 0.0, 0.0,  -1.0, 0.0, 0.0,  -1.0, 0.0, 0.0,
				 0.0,-1.0, 0.0,   0.0,-1.0, 0.0,   0.0,-1.0, 0.0,   0.0,-1.0, 0.0,
				 0.0, 0.0,-1.0,   0.0, 0.0,-1.0,   0.0, 0.0,-1.0,   0.0, 0.0,-1.0
			]);
			this.normals.units = this.normals.length / (this.normals.size = 3);
			return this;
		};
		obj.giveColors = function () {
			if (this.colors) return this;
			this.colors = new Float32Array([
				0.0, 0.0, 1.0, 1.0,   0.0, 0.0, 1.0, 1.0,   0.0, 0.0, 1.0, 1.0,   0.0, 0.0, 1.0, 1.0,
				1.0, 0.0, 0.0, 1.0,   1.0, 0.0, 0.0, 1.0,   1.0, 0.0, 0.0, 1.0,   1.0, 0.0, 0.0, 1.0,
				0.0, 1.0, 0.0, 1.0,   0.0, 1.0, 0.0, 1.0,   0.0, 1.0, 0.0, 1.0,   0.0, 1.0, 0.0, 1.0,
				1.0, 1.0, 0.0, 1.0,   1.0, 1.0, 0.0, 1.0,   1.0, 1.0, 0.0, 1.0,   1.0, 1.0, 0.0, 1.0,
				1.0, 0.0, 1.0, 1.0,   1.0, 0.0, 1.0, 1.0,   1.0, 0.0, 1.0, 1.0,   1.0, 0.0, 1.0, 1.0,
				0.0, 1.0, 1.0, 1.0,   0.0, 1.0, 1.0, 1.0,   0.0, 1.0, 1.0, 1.0,   0.0, 1.0, 1.0, 1.0
			]);
			this.colors.units = this.colors.length / (this.colors.size = 4);
			return this;
		};
		obj.load = function (cb) {
			if (this.textureUrl) {
				this.texture = this.world.webgl.loadTexture(this.textureUrl, cb);
			} else {
				util.safeApply(cb, this);
			}
			return this;
		};
		obj.buffer = function () {
			var webgl = this.world.webgl;
			this.unbuffer();
			if (this.indices) this.indexBuf = webgl.createElementArrayBuffer(this.indices, true);
			if (this.points) this.pointsBuf = webgl.createArrayBuffer(this.points, true);
			if (this.normals) this.normalBuf = webgl.createArrayBuffer(this.normals, true);
			if (this.colors) this.colorBuf = webgl.createArrayBuffer(this.colors, true);
			return this;
		};
		obj.unbuffer = function () {
			var gl = this.world.gl;
			if (this.indexBuf) gl.deleteBuffer(this.indexBuf);
			if (this.pointsBuf) gl.deleteBuffer(this.pointsBuf);
			if (this.normalBuf) gl.deleteBuffer(this.normalBuf);
			if (this.colorBuf) gl.deleteBuffer(this.colorBuf);
			return this;
		};
		obj.render = function (webgl, gl) {
			return supr.render.call(this, function () {
				var attrs = this.shader.attrs;

				if (this.pointsBuf) {
					gl.enableVertexAttribArray(attrs.position);
					gl.bindBuffer(gl.ARRAY_BUFFER, this.pointsBuf);
					gl.vertexAttribPointer(attrs.position, this.points.vertexSize, gl.FLOAT, false, 20, 0);
					if (this.texture) {
						gl.enableVertexAttribArray(attrs.texCoord);
						gl.vertexAttribPointer(attrs.texCoord, this.points.texCoordSize, gl.FLOAT, false, 20, 12);
					}
				}

				if (this.normalBuf) {
					gl.enableVertexAttribArray(attrs.normal);
					gl.bindBuffer(gl.ARRAY_BUFFER, this.normalBuf);
					gl.vertexAttribPointer(attrs.normal, this.normals.size, gl.FLOAT, false, 0, 0);
				}

				if (this.colorBuf) {
					gl.enableVertexAttribArray(attrs.color);
					gl.bindBuffer(gl.ARRAY_BUFFER, this.colorBuf);
					gl.vertexAttribPointer(attrs.color, this.colors.size, gl.FLOAT, false, 0, 0);
				}

				gl.bindBuffer(gl.ARRAY_BUFFER, null);

				if (this.pointsBuf && this.texture) {
					gl.bindTexture(gl.TEXTURE_2D, this.texture);
				}

				gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.indexBuf);
				gl.drawElements(gl.TRIANGLES, this.indices.length, gl.UNSIGNED_BYTE, 0);

				gl.bindBuffer(gl.ELEMENT_ARRRAY_BUFFER, null);
			});
		};
		if (args) obj.initCube.apply(obj, args);
		return obj;
	};
	(module.exports = Cube).create(cube);
}).call(this);